# PA165 Purple Team
Spring 2023, MUNI

# About Project
- **Name:** Ice-Hockey Manager
- **Technologies:** Java 17, Spring, Maven, Tomcat
- **Developer IDs (UCO):**
    - 492777
    - 492988 (Supreme Chancellor)
    - 485115
    - 540468
- **Assignment:** Several human players (at least two) can manage their hockey teams out of a list of real ice hockey teams of several championships across Europe. Human players can pick their team and add / remove ice hockey players from a list of available free agents. There is a schedule of games and results will be generated taking into account the players characteristics (no need to have some advanced algorithm simulating games: just a simple randomization will do it!). Admin can put new hockey players in the main list of free agents and change their attributes before they are selected by other human players. If you want, you can implement a budget system for each team, so that players can be bought and sold based on the financial availability of teams.

# How to Run
Note: The latest version of docker comes with `docker compose`, so you don't need to install it separately. On the other hand, it is possible to download `docker-compose` command (which is a separate tool) and use it instead of `docker compose`.
```bash
docker compose up     # will start all services and create a network between them, optional -d flag for detached mode

# to stop the services
docker compose down --volumes   # will stop amd remove all containers
```

# Scenarios  
In our project we created two main scenarios (that should cover most of the functionality) with help of locust (https://locust.io/). These scenarios are simple python scripts which are using the rest endpoints and manipulating with the responses. We will provide brief description of each scenario and at the end there will also be a manual how to run them.  
Note: before running the scenarios, it is recommended to run also the database seeding as the data are required for scenarios  

## First scenario - Manager
There will be x (max 20) users or managers which should do the following actions:  
- Choose a random name and surname and register your user to core service
- Create your own team (again with some random concatenation of strings)
- Add 20 players to your team (for each player add a comment to discussion forum)
- Enroll your team to one of the leagues
- Create arena reservations for trainings  
In this scenario, we expect more load, since there are multiple users or tasks calling endpoints each second.

## Second scenario - Admin
The second scenario will be slightly dependent on the first one. After the users or managers created their teams, now it is time for the admin (only 1) to step in and do the following actions:
- List all leagues
- For each league create schedule, games and create also reservations for each of the games
- After each league was iterated through, create and notify specific subscriber about newly created game(s)  
This scenario is showing more functionality than simulating some heavy load on the system.

## How to run scenarios
There are two files in the following [directory](https://gitlab.fi.muni.cz/xpaluba/pa165-ice-hockey-manager/-/tree/main/locust-scenarios). Each of them has to be run separately and we recommended starting with the manager scenario.

Firstly create a python virtual environment and install required packages
```bash
python3 -m venv venv # you might be promted to install python3-venv, go ahead and do it
source venv/bin/activate # activate the virtual environment
pip install -r locust-scenarios/requirements.txt # install required packages
```

The seeding step is for the scenarios mandatory, so refer to seeding section, but before proceeding, we want to use different docker-compose file.  
Use the following when starting lines when starting the services.
```bash
docker compose -f compose.yaml -f compose-insecure.yaml up -d  # start all services in insecure mode (a lot easier for scenario)
# wait for all services to initialize DBs before seeding; you can check it at Prometheus http://localhost:9090/targets (wait for all green OK)

./utils/seeding-scripts/sql_scripts_exec.sh 
# to seed the database (scenario requires some initial data); there is also a .ps1 script for windows called sql_scripts_exec.ps1
# (in case you do not want to scroll or not properly following our guide)
```

Lastly we will need the locust and specified files. Now for each of the two scenarios run the following:
```
locust -f locust_manager.py         # for admin locust_admin.py
# open browser and http://localhost:8089
# input requred fields (url == http://localhost:8080) and start swarming
# remember up to 20 for manager and only 1 user per admin, otherwise the scenario does not make sense
```
Check grafana or the http://localhost:8089 for statistics and which endpoints were called.

The execution ot the whole scenario will take a few minutes. After it is finished, you can see the results in the web interface.

The monitoring statistics are available to you in already running Prometheus and Grafana services.
Grafana is running at http://localhost:3000, and you can log in with admin/admin credentials (you will be promoted to change the password, go ahead and skip it).
Open the menu (top left) and go to Dashboards. In folder `General` you can see 2 dashboards monitoring all services.


# Database seeding (optional step)  
Database seeding is an optional step, which requires .sql files and powershell/bash scripts (found in the following [directory](https://gitlab.fi.muni.cz/xpaluba/pa165-ice-hockey-manager/-/tree/main/utils)). Each of the .sql scripts contains some dummy data (consider adding new records) and could be executed with the help of scripts. We provided powershell and also bash scripts, so feel free to choose the one most suitable for your environment.  
Note: In the scripts we are using podman, but it is hidden under alias, so consider changing only the tool name from podman to docker.
```
docker compose down --volumes # will stop and remove all containers and volumes, important before seeding the database

docker compose up -d
# wait for all services initialize DBs; you can check it at Prometheus http://localhost:9090/targets (wait for all green OK)
# or specify different file for scenarios with --file


# For bash script
./utils/seeding-scripts/sql_scripts_exec.sh      # run from root directory

# For powershell script
Get-ExecutionPolicy                             # check if it is already Unrestricted
Set-ExecutionPolicy Unrestricted                # switch to Unrestricted mode
./utils/seeding-scripts/sql_scripts_exec.ps1    # run from root directory
# consider changing the Unrestricted policy back

```

# Project Modules
In our project we introduced 4 separate modules and each of them represent different service. Each service is a spring boot application with the n-tier architecture starting from data layer all the way up to the rest controllers.  
We introduced following modules (with brief description):
- **Hockey-manager core service** - Core application logic
- **Discussion forum service** - Discussion platform of the application
- **Reporting service** - Responsible for sending notification e-mails about various events to subscribed users
- **Arena reservation service** - Handling of arena reservations for different hockey league matches, training etc.  

# Module description
This section provides more detailed description of each service with some additional diagrams for better service understanding.
## I. Hockey-Manager Service
 * **Description:** This service is the core of the whole application. It is responsible for managing the whole hockey team, players, matches, leagues etc. It is also responsible for the communication with the other services (arena reservation, discussion forum, reporting).

### Use Case Diagram
![Hockey-Manager Use Case Diagram](hockey-manager-service/diagrams/use_case_diagram.png)

### Class Diagram
![Hockey-Manager Class Diagram](hockey-manager-service/diagrams/class_diagram.png)

### Entity-relationship Diagram
![ERD CORE](hockey-manager-service/diagrams/ERD_CORE.png)

## II. Discussion Forum Service
 * **Description:** This service is responsible for the communication between the users of the application. It is used for the discussion of the hockey matches, players, teams etc. It is also used for the communication between the users and the administrators of the application.

### Use Case Diagram
![Discussion Use Case Diagram](discussion-forum-service/diagrams/use_case_diagram.png)

### Class Diagram
![Discussion Class Diagram](discussion-forum-service/diagrams/class_diagram.png)

### Entity-relationship Diagram
![ERD CORE](discussion-forum-service/diagrams/ERD_DISCUSSION.png)

## III. Reporting Service
 * **Description:** This service is responsible for notifying the users of the results of the matches and possibly other events.

### Use Case Diagram
![Report Use Case Diagram](report-service/diagrams/report_service_use_case_diagram.png)

### Class Diagram
![Report Class Diagram](report-service/diagrams/report_service_class_diagram.png)

### Entity-relationship Diagram
![ERD CORE](report-service/diagrams/ERD_REPORT.png)

## IV. Arena Reservation Service
 * **Description:** This service is simple application used mainly for managing few actions tied to the reservation of specific arena for ice-hockey matches or trainings.

### Use Case Diagram
It enables the system user to list existing arenas, check the timetable for each of the arena and make a reservation, in case he/she is interested in specific arena. He/she can later change his/her mind, modify the reservation or even cancel it completely (with respect to the time).  
The second role in this system is administrator, which is responsible for adding, updating or even deleting existing arenas from this service.  
All this functionality can also be seen at the use case diagram below.  

![Arena Use Case Diagram](arena-reservation-service/diagrams/ARS_UC.PNG)

### Class Diagram
During the analysis phase we also discovered some useful data objects, which will be used in the service and stored in the database. We need to represent the reservation object as well as the arena itself (tied to some specific physical place in real world). There are a lot of possibilities, but we want to keep things simple and add only the required and logical attributes or features (described on the class diagram below).

![Arena Class Diagram](arena-reservation-service/diagrams/ARS_CD.PNG)

### Entity-relationship Diagram
![ERD CORE](arena-reservation-service/diagrams/ERD_ARENA.png)

# Milestone 1 requirements
In this section we will introduce some comments to the first milestone requirements.
- Our teamleader created a public repository, [main page](https://gitlab.fi.muni.cz/xpaluba/pa165-ice-hockey-manager)
- We organized the README file to include all necessary information, mainly the service description and how to run sections, diagrams etc.
- We created 4 services (core service, discussion forum, reporting and arena reservation), each implemented as spring boot application, for each service we also created rest api with some dummy data logic underneath (also called stubs)
- After adding some dependencies and with right annotation we could easily generate openapi files
- With help of Mockito we managed to create a few unit test for each of the 4 services
- The progress of our work can be seen directly on gitlab as issues, (merged) branches or even individual commits

# Milestone 2 requirements
- First of all, we revisited and implemented feedback from project evaluations. Besides that, we also refactored and improved several
other things from Milestone 1.
- Added persistence layer for core service and report service (9 entities in total). 
We transitioned to using lombok in entities for generating constructors, getter+setters, equals+hashcode, etc.
- We extended tests for all layers in core service and report service.
- For persistence, we have a configuration for both H2 in-memory database (for development) and PostgreSQL database (for production).
- Our two required business functionalities include starting+simulating the game and generating the schedule of games for a league.
- Dockerization of all microservices, along with PostgreSQL.
- We already created Gitlab pipeline in Milestone 1. 
In M2, we decided to create and use our own custom Gitlab pipeline runner.
- Global exception handling as a nice new feature

# Milestone 3 requirements
- Same as in the second milestone, the first step was to recheck and correct everything relevant from the feedback.
- The rest endpoints were already connected to the persistence layer in the second milestone, but we made few modifications to improve the code.
- OAuth 2 or security was implemented it almost all services, with scope "ADMIN" and only authorized people can access most of the endpoints (authorization through MUNI).
- Exposure of actuator health and metrics was done for all services.
- Prometheus will run in separate docker container (http://localhost:9090) and it will automatically gather relevant statistics about the system.
- In another docker container Grafana should be running (http://localhost:3000), where we also provided datasource (Prometheus) and 2 already preconfigured dashboards.
- We defined and implemented 2 runnable scenarios, one which is heavier on load where up to 20 people/tasks can be executed in parallel and the second scenario that is more dedicated to show functionality.
- We create .sql scripts for database seeding. In order to enlighten the workload with seeding, we decided to create bash and powershell scripts to automatically seed the database.
