package cz.muni.fi.icehockeymanager.arenareservation.data.domain;

import jakarta.persistence.*;
import jakarta.validation.constraints.NotNull;
import lombok.*;

import java.io.Serializable;
import java.util.Objects;


@Getter
@Setter
@NoArgsConstructor
@AllArgsConstructor
@Entity
@Table(name = "address")
public class Address implements Serializable {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    @NotNull
    @Column(length = 100)
    private String street;

    @NotNull
    @Column(length = 30)
    private String streetNumber;

    @NotNull
    @Column(length = 40)
    private String city;

    @Column(length = 40)
    private String region;

    @NotNull
    @Column(length = 3)
    private String countryISO;

    @OneToOne(mappedBy = "address")
    private Arena arena;

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (!(obj instanceof Address addressCmp)) {
            return false;
        }
        return (Objects.equals(getStreet(), addressCmp.getStreet()) &&
                Objects.equals(getStreetNumber(), addressCmp.getStreetNumber()) &&
                Objects.equals(getCity(), addressCmp.getCity()) &&
                Objects.equals(getCountryISO(), addressCmp.getCountryISO()));
    }
    @Override
    public int hashCode() {
        return Objects.hash(getStreet(), getCity(), getCountryISO());
    }

    @Override
    public String toString() {
        return "Address{" +
                "id=" + id +
                ", street='" + street  + '\'' +
                ", streetNumber='" + streetNumber + '\'' +
                ", city='" + city + '\'' +
                ", region='" + region + '\'' +
                ", country='" + countryISO + '\'' +
                "}";
    }
}