package cz.muni.fi.icehockeymanager.arenareservation.data.domain;


import jakarta.persistence.*;
import jakarta.validation.constraints.Min;
import jakarta.validation.constraints.NotNull;
import lombok.*;

import java.io.Serializable;
import java.util.Objects;
import java.util.Set;

@Getter
@Setter
@NoArgsConstructor
@AllArgsConstructor
@Entity
@Table(name = "arena")
public class Arena implements Serializable {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    @OneToOne(cascade = CascadeType.ALL, fetch = FetchType.EAGER)
    @JoinColumn(name = "address_id")
    private Address address;

    @NotNull
    @Column(length = 50)
    private String arenaName;

    @NotNull
    @Min(1950)
    private int yearOfConstruction;

    private int capacity;

    private double dimensions;

    @OneToMany(mappedBy = "arena", cascade = CascadeType.ALL, orphanRemoval = true)
    private Set<ReservationRecord> reservationRecords;

    @Override
    public int hashCode() {
        return Objects.hash(getArenaName(), getYearOfConstruction());
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (!(obj instanceof Arena arenaCmp)) {
            return false;
        }
        return (Objects.equals(getArenaName(), arenaCmp.getArenaName()) &&
                Objects.equals(getAddress(), arenaCmp.getAddress()) &&
                Objects.equals(getYearOfConstruction(), arenaCmp.getYearOfConstruction()));
    }

    @Override
    public String toString() {
        return "Arena{" +
                "id=" + id +
                ", name='" + arenaName  + '\'' +
                ", yearOfConstruction=" + yearOfConstruction +
                ", capacity=" + capacity +
                ", dimensions=" + dimensions +
                "}";
    }
}
