package cz.muni.fi.icehockeymanager.arenareservation.data.domain;

import jakarta.persistence.*;
import jakarta.validation.constraints.Email;
import jakarta.validation.constraints.NotNull;
import lombok.*;
import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;

import java.io.Serializable;
import java.util.Objects;
import java.util.Set;

@Getter
@Setter
@NoArgsConstructor
@AllArgsConstructor
@Entity
@Table(name = "contact_person")
public class ContactPerson implements Serializable {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    @NotNull
    @Column(length = 50)
    private String name;

    @NotNull
    @Column(length = 50)
    private String surname;

    @NotNull
    @Email
    @Column(length = 120)
    private String email;

    @NotNull
    @Column(length = 30)
    private String telephoneNumber;

    @OneToMany(mappedBy = "contactPerson", cascade = CascadeType.ALL, orphanRemoval = true)
    private Set<ReservationRecord> reservationRecords;

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (!(obj instanceof ContactPerson personCmp)) {
            return false;
        }
        return (Objects.equals(getName(), personCmp.getName()) &&
                Objects.equals(getSurname(), personCmp.getSurname()) &&
                Objects.equals(getEmail(), personCmp.getEmail()));
    }
    @Override
    public int hashCode() {
        return Objects.hash(getName(), getSurname(), getEmail());
    }

    @Override
    public String toString() {
        return "ContactPerson{" +
                "id=" + id +
                ", name='" + name  + '\'' +
                ", surname='" + surname + '\'' +
                ", email='" + email + '\'' +
                ", telephoneNumber='" + telephoneNumber + '\'' +
                "}";
    }
}
