package cz.muni.fi.icehockeymanager.arenareservation.data.domain;

import cz.muni.fi.icehockeymanager.arenareservation.data.enums.ReservationType;
import jakarta.persistence.*;
import jakarta.validation.constraints.Future;
import jakarta.validation.constraints.FutureOrPresent;
import jakarta.validation.constraints.NotNull;
import lombok.*;

import java.io.Serializable;
import java.sql.Timestamp;
import java.util.Objects;

@Getter
@Setter
@NoArgsConstructor
@AllArgsConstructor
@Entity
@Table(name = "reservation")
public class ReservationRecord implements Serializable {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "arena_id")
    private Arena arena;

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "person_id")
    private ContactPerson contactPerson;

    @NotNull
    @FutureOrPresent
    @Temporal(value = TemporalType.TIMESTAMP)
    private Timestamp dateTimeFrom;

    @NotNull
    @Future
    @Temporal(value = TemporalType.TIMESTAMP)
    private Timestamp dateTimeTo;

    @Enumerated(EnumType.STRING)
    @Column(length = 15)
    private ReservationType reservationType;

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (!(obj instanceof ReservationRecord recordCmp)) {
            return false;
        }
        return (Objects.equals(getArena(), recordCmp.getArena()) &&
                Objects.equals(getContactPerson(), recordCmp.getContactPerson()) &&
                Objects.equals(getDateTimeFrom(), recordCmp.getDateTimeFrom()) &&
                Objects.equals(getDateTimeTo(), recordCmp.getDateTimeTo()) &&
                Objects.equals(getReservationType(), recordCmp.getReservationType()));
    }
    @Override
    public int hashCode() {
        return Objects.hash(getArena(), getContactPerson(), getDateTimeFrom(), getDateTimeTo());
    }

    @Override
    public String toString() {
        return "ReservationRecord{" +
                "id=" + id +
                ", dateTimeFrom='" + dateTimeFrom  + '\'' +
                ", dateTimeTo='" + dateTimeTo + '\'' +
                ", reservationType='" + reservationType + '\'' +
                "}";
    }
}
