package cz.muni.fi.icehockeymanager.arenareservation.data.repository;

import cz.muni.fi.icehockeymanager.arenareservation.data.domain.ReservationRecord;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.stereotype.Repository;

import java.sql.Timestamp;
import java.util.List;

@Repository
public interface ReservationRepository extends JpaRepository<ReservationRecord, Long> {

    @Query("SELECT r FROM ReservationRecord r " +
            "WHERE r.contactPerson.id = :userId " +
            "AND r.dateTimeFrom >= :fromDateTime " +
            "AND r.dateTimeTo <= :toDateTime")
    List<ReservationRecord> findAllWithUserIdAndDateFiltering(
            @Param("userId") Long userID,
            @Param("fromDateTime") @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) Timestamp fromDateTime,
            @Param("toDateTime") @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) Timestamp toDateTime);

    @Query("SELECT r FROM ReservationRecord r " +
            "WHERE r.arena.id = :arenaId " +
            "AND r.dateTimeFrom >= :fromDateTime " +
            "AND r.dateTimeTo <= :toDateTime")
    List<ReservationRecord> findAllByArenaIdWithDateFiltering(
            @Param("arenaId") Long arenaID,
            @Param("fromDateTime") @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) Timestamp fromDateTime,
            @Param("toDateTime") @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) Timestamp toDateTime);

    @Query("SELECT count(r) FROM ReservationRecord r " +
            "WHERE r.dateTimeTo >= :fromDateTime " +
            "AND r.dateTimeFrom <= :toDateTime " +
            "AND r.arena.id = :arenaId")
    int timeCollisionExist(
            @Param("arenaId") Long arenaID,
            @Param("fromDateTime") @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) Timestamp fromDateTime,
            @Param("toDateTime") @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) Timestamp toDateTime);
}