package cz.muni.fi.icehockeymanager.arenareservation.facade;

import cz.muni.fi.icehockeymanager.arenareservation.api.edit.EditArenaDto;
import cz.muni.fi.icehockeymanager.arenareservation.api.view.ArenaDto;
import cz.muni.fi.icehockeymanager.arenareservation.api.view.ArenaDetailDto;
import cz.muni.fi.icehockeymanager.arenareservation.api.create.CreateArenaDto;
import cz.muni.fi.icehockeymanager.arenareservation.mappers.ArenaMapper;
import cz.muni.fi.icehockeymanager.arenareservation.service.ArenaService;
import jakarta.transaction.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;


import java.util.List;

@Service
@Transactional
public class ArenaFacade {

    private final ArenaService arenaService;
    private final ArenaMapper arenaMapper;

    @Autowired
    public ArenaFacade(ArenaService arenaService, ArenaMapper arenaMapper) {
        this.arenaService = arenaService;
        this.arenaMapper = arenaMapper;
    }

    public List<ArenaDto> getAllArenas() {
        return arenaMapper.mapArenaListToArenaDtoList(arenaService.getAllArenas());
    }

    public ArenaDetailDto getArenaDetailView(Long arenaID) {
        return arenaMapper.mapArenaEntityToArenaDetailDto(
                arenaService.findArenaById(arenaID));
    }

    public ArenaDetailDto createArena(CreateArenaDto createArenaDTO) {
        return arenaMapper.mapArenaEntityToArenaDetailDto(
                arenaService.saveArena(
                        arenaMapper.mapCreateArenaDtoToArenaEntity(createArenaDTO)));
    }

    public ArenaDetailDto modifyArena(Long arenaID, EditArenaDto editArenaDto) {
        ArenaDetailDto arenaDetailDto = getArenaDetailView(arenaID);
        if (!editArenaDto.getArenaName().isEmpty())
            arenaDetailDto.setArenaName(editArenaDto.getArenaName());
        if (editArenaDto.getCapacity() != 0)
            arenaDetailDto.setCapacity(editArenaDto.getCapacity());
        arenaService.saveArena(arenaMapper.mapArenaDetailDtoToArenaEntity(arenaDetailDto));
        return arenaDetailDto;
    }

    public void deleteArena(Long arenaID) {
        arenaService.deleteArena(arenaID);
    }
}
