package cz.muni.fi.icehockeymanager.arenareservation.rest;

import cz.muni.fi.icehockeymanager.arenareservation.api.edit.EditArenaDto;
import cz.muni.fi.icehockeymanager.arenareservation.api.view.ArenaDto;
import cz.muni.fi.icehockeymanager.arenareservation.api.view.ArenaDetailDto;
import cz.muni.fi.icehockeymanager.arenareservation.api.create.CreateArenaDto;
import cz.muni.fi.icehockeymanager.arenareservation.facade.ArenaFacade;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import java.util.List;

@RestController
@RequestMapping("/arenas")
public class ArenaRestController {

    private final ArenaFacade arenaFacade;

    @Autowired
    public ArenaRestController(ArenaFacade arenaFacade) {
        this.arenaFacade = arenaFacade;
    }

    @Operation(summary = "List arenas")
    @GetMapping
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<List<ArenaDto>> getArenas() {
        return ResponseEntity.ok(arenaFacade.getAllArenas());
    }

    @Operation(summary = "Get Detail view of Arena")
    @GetMapping("/{arenaID}")
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<ArenaDetailDto> getArenaDetailView(@PathVariable("arenaID") Long arenaID) {
        return ResponseEntity.ok(arenaFacade.getArenaDetailView(arenaID));
    }

    @Operation(
        summary = "Create new Arena",
        security = @SecurityRequirement(name = "Bearer")
    )
    @PostMapping()
    @ResponseStatus(HttpStatus.CREATED)
    public ResponseEntity<ArenaDetailDto> createArena(@RequestBody CreateArenaDto createArenaDTO) {
        return new ResponseEntity<>(arenaFacade.createArena(createArenaDTO), HttpStatus.CREATED);
    }

    @Operation(
        summary = "Modify existing arena",
        security = @SecurityRequirement(name = "Bearer")
    )
    @PatchMapping("/{arenaID}")
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<ArenaDetailDto> modifyArena(@PathVariable Long arenaID,
                                                      @RequestBody EditArenaDto editArenaDto) {
        return ResponseEntity.ok(arenaFacade.modifyArena(arenaID, editArenaDto));
    }

    @Operation(
        summary = "Delete existing arena",
        security = @SecurityRequirement(name = "Bearer")
    )
    @DeleteMapping("/{arenaID}")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    public ResponseEntity<ArenaDto> deleteArena(@PathVariable("arenaID") Long arenaID) {
        arenaFacade.deleteArena(arenaID);
        return ResponseEntity.noContent().build();
    }
}
