package cz.muni.fi.icehockeymanager.arenareservation.rest;

import cz.muni.fi.icehockeymanager.arenareservation.api.create.CreateContactPersonDto;
import cz.muni.fi.icehockeymanager.arenareservation.api.view.ContactPersonDetailDto;
import cz.muni.fi.icehockeymanager.arenareservation.facade.ContactPersonFacade;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@RestController
@SecurityRequirement(name = "Bearer")
@RequestMapping("/people")
public class ContactPersonRestController {
    private final ContactPersonFacade contactPersonFacade;

    @Autowired
    public ContactPersonRestController(ContactPersonFacade contactPersonFacade) {
        this.contactPersonFacade = contactPersonFacade;
    }

    @Operation(summary = "Register new Contact Person")
    @PostMapping()
    @ResponseStatus(HttpStatus.CREATED)
    public ResponseEntity<ContactPersonDetailDto> registerNewContactPerson(
            @RequestBody CreateContactPersonDto createContactPersonDto) {
        return new ResponseEntity<>(contactPersonFacade.createContactPerson(createContactPersonDto), HttpStatus.CREATED);
    }

    @Operation(summary = "Delete Contact Person")
    @DeleteMapping("/{contactPersonId}")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    public ResponseEntity<Object> deleteContactPerson(@PathVariable("contactPersonId") Long contactPersonId) {
        contactPersonFacade.deleteContactPerson(contactPersonId);
        return ResponseEntity.noContent().build();
    }
}
