package cz.muni.fi.icehockeymanager.arenareservation.rest;

import cz.muni.fi.icehockeymanager.arenareservation.api.edit.EditArenaReservationRecordDto;
import cz.muni.fi.icehockeymanager.arenareservation.api.view.ArenaReservationRecordDetailDto;
import cz.muni.fi.icehockeymanager.arenareservation.api.create.CreateArenaReservationRecordDto;
import cz.muni.fi.icehockeymanager.arenareservation.api.view.ArenaReservationRecordDto;
import cz.muni.fi.icehockeymanager.arenareservation.facade.ReservationFacade;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.List;

@RestController
@RequestMapping("/reservations")
public class ReservationRestController {
    private final ReservationFacade reservationFacade;

    @Autowired
    public ReservationRestController(ReservationFacade reservationFacade) {
        this.reservationFacade = reservationFacade;
    }

    @Operation(summary = "Get Timetable of reservations for arena")
    @GetMapping("/arenas/{arenaID}")
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<List<ArenaReservationRecordDto>> getTimetableForArena(
            @PathVariable("arenaID") Long arenaID,
            @RequestParam(name = "from", required = false) @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) LocalDateTime fromDateTime,
            @RequestParam(name = "to", required = false) @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) LocalDateTime toDateTime) {
        return ResponseEntity.ok(reservationFacade.getTimetableForArena(arenaID, fromDateTime, toDateTime));
    }

    @Operation(summary = "Get Reservation for specific Contact Person")
    @GetMapping("/{contactPersonId}")
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<List<ArenaReservationRecordDetailDto>> getReservationsByPersonId(
            @PathVariable("contactPersonId") Long contactPersonId,
            @RequestParam(name = "from", required = false) @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) LocalDateTime fromDateTime,
            @RequestParam(name = "to", required = false) @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) LocalDateTime toDateTime) {
        return ResponseEntity.ok(reservationFacade.getReservationsByPersonId(contactPersonId, fromDateTime, toDateTime));
    }

    @Operation(
        summary = "Create new reservation",
        security = @SecurityRequirement(name = "Bearer")
    )
    @PostMapping("/{arenaID}")
    @ResponseStatus(HttpStatus.CREATED)
    public ResponseEntity<ArenaReservationRecordDetailDto> createNewReservation(
            @PathVariable("arenaID") Long arenaID,
            @RequestBody CreateArenaReservationRecordDto record) {
        return new ResponseEntity<>(reservationFacade.createNewReservation(arenaID, record), HttpStatus.CREATED);
    }

    @Operation(
        summary = "Modify reservation record",
        security = @SecurityRequirement(name = "Bearer")
    )
    @PatchMapping("/{recordID}")
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<ArenaReservationRecordDetailDto> modifyReservation(
            @PathVariable("recordID") Long recordID, @RequestBody EditArenaReservationRecordDto record) {
        return ResponseEntity.ok(reservationFacade.modifyReservation(recordID, record));
    }

    @Operation(
        summary = "Delete reservation record",
        security = @SecurityRequirement(name = "Bearer")
    )
    @DeleteMapping("/{recordID}")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    public ResponseEntity<ArenaReservationRecordDetailDto> deleteReservation(@PathVariable("recordID") Long recordID) {
        reservationFacade.deleteReservation(recordID);
        return ResponseEntity.noContent().build();
    }
}
