package cz.muni.fi.icehockeymanager.arenareservation.rest.restexceptionhandling;

import cz.muni.fi.icehockeymanager.arenareservation.exceptions.ResourceNotFoundException;
import cz.muni.fi.icehockeymanager.arenareservation.exceptions.TimeTableCollisionException;
import cz.muni.fi.icehockeymanager.arenareservation.rest.ArenaRestController;
import cz.muni.fi.icehockeymanager.arenareservation.rest.ContactPersonRestController;
import cz.muni.fi.icehockeymanager.arenareservation.rest.ReservationRestController;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.context.request.WebRequest;
import org.springframework.web.method.annotation.MethodArgumentTypeMismatchException;
import org.springframework.web.servlet.mvc.method.annotation.ResponseEntityExceptionHandler;

import java.util.NoSuchElementException;


@ControllerAdvice(
        assignableTypes = {
                ArenaRestController.class,
                ReservationRestController.class,
                ContactPersonRestController.class
        })
public class ArenaReservationExceptionHandling extends ResponseEntityExceptionHandler {

    public ArenaReservationExceptionHandling() {
        super();
    }

    @ExceptionHandler({MethodArgumentTypeMismatchException.class})
    public ResponseEntity<Object> handleMethodArgumentMismatch(final MethodArgumentTypeMismatchException exception,
                                                               final WebRequest request) {
        final String responseBody = "Client-Side Error: " + exception.getLocalizedMessage();
        return handleExceptionInternal(exception, responseBody,
                new HttpHeaders(), HttpStatus.BAD_REQUEST, request);
    }

    @ExceptionHandler({TimeTableCollisionException.class})
    public ResponseEntity<Object> handleTimeTableCollision(final TimeTableCollisionException exception,
                                                               final WebRequest request) {
        final String responseBody = "Client-Side Error: " + exception.getLocalizedMessage();
        return handleExceptionInternal(exception, responseBody,
                new HttpHeaders(), HttpStatus.BAD_REQUEST, request);
    }

    @ExceptionHandler({ResourceNotFoundException.class})
    public ResponseEntity<Object> handleResourceNotFound(final ResourceNotFoundException exception,
                                                            final WebRequest request) {
        final String responseBody = "Client-Side Error" + exception.getLocalizedMessage();
        return handleExceptionInternal(exception, responseBody,
                new HttpHeaders(), HttpStatus.NOT_FOUND, request);
    }

    @ExceptionHandler({ NullPointerException.class, IllegalArgumentException.class,
            IllegalStateException.class, NoSuchElementException.class })
    public ResponseEntity<Object> handleInternal(final RuntimeException exception, final WebRequest request) {
        final String responseBody = "Server-Side Error: " + exception.getLocalizedMessage();
        return handleExceptionInternal(exception, responseBody,
                new HttpHeaders(), HttpStatus.INTERNAL_SERVER_ERROR, request);
    }
}
