package cz.muni.fi.icehockeymanager.arenareservation.service;

import cz.muni.fi.icehockeymanager.arenareservation.data.domain.Arena;
import cz.muni.fi.icehockeymanager.arenareservation.data.repository.AddressRepository;
import cz.muni.fi.icehockeymanager.arenareservation.data.repository.ArenaRepository;
import cz.muni.fi.icehockeymanager.arenareservation.exceptions.ResourceNotFoundException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import java.util.List;

@Service
@Transactional
public class ArenaService {
    private final ArenaRepository arenaRepository;
    private final AddressRepository addressRepository;
    @Autowired
    public ArenaService(ArenaRepository arenaRepository, AddressRepository addressRepository) {
        this.arenaRepository = arenaRepository;
        this.addressRepository = addressRepository;
    }

    public List<Arena> getAllArenas() {
        return arenaRepository.findAll();
    }

    public Arena findArenaById(Long arenaId) {
        return arenaRepository.findById(arenaId)
                .orElseThrow(
                        () -> new ResourceNotFoundException("Arena with specified ID (" + arenaId + ") was not found")
                );
    }

    public Arena saveArena(Arena arena) {
        return arenaRepository.save(arena);
    }

    public void deleteArena(Long arenaId) {
        arenaRepository.deleteById(arenaId);
    }

    public void deleteAll() {
        arenaRepository.deleteAll();
    }
}
