package cz.muni.fi.icehockeymanager.arenareservation.service;

import cz.muni.fi.icehockeymanager.arenareservation.data.domain.ContactPerson;
import cz.muni.fi.icehockeymanager.arenareservation.data.repository.ContactPersonRepository;
import cz.muni.fi.icehockeymanager.arenareservation.exceptions.ResourceNotFoundException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.nio.file.ReadOnlyFileSystemException;

@Transactional
@Service
public class ContactPersonService {
    private final ContactPersonRepository contactPersonRepository;

    @Autowired
    public ContactPersonService(ContactPersonRepository contactPersonRepository) {
        this.contactPersonRepository = contactPersonRepository;
    }

    public ContactPerson save(ContactPerson contactPerson) {
        return contactPersonRepository.save(contactPerson);
    }

    public ContactPerson findById(Long contactPersonId) {
        return contactPersonRepository.findById(contactPersonId)
                .orElseThrow(
                        () -> new ResourceNotFoundException("Contact Person with specified ID ("
                                + contactPersonId + ") was not found")
                );
    }

    public void deleteById(Long contactPersonId) {
        contactPersonRepository.deleteById(contactPersonId);
    }

    public void deleteAll() {
        contactPersonRepository.deleteAll();
    }
}
