package cz.muni.fi.icehockeymanager.arenareservation.service;

import cz.muni.fi.icehockeymanager.arenareservation.data.domain.ReservationRecord;
import cz.muni.fi.icehockeymanager.arenareservation.data.repository.ReservationRepository;
import cz.muni.fi.icehockeymanager.arenareservation.exceptions.ResourceNotFoundException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import java.sql.Timestamp;
import java.util.List;

@Service
@Transactional
public class ReservationService {

    private final ReservationRepository reservationRepository;

    @Autowired
    public ReservationService(ReservationRepository reservationRepository) {
        this.reservationRepository = reservationRepository;
    }

    public ReservationRecord saveReservationRecord(ReservationRecord reservationRecord) {
        return reservationRepository.save(reservationRecord);
    }

    public List<ReservationRecord> getAllRecordsByUserId(Long userID, Timestamp fromDateTime, Timestamp toDateTime) {
        return reservationRepository.findAllWithUserIdAndDateFiltering(userID, fromDateTime, toDateTime);
    }

    public List<ReservationRecord> getAllRecordsByArenaId(Long arenaId, Timestamp fromDateTime,
                                                          Timestamp toDateTime) {
        return reservationRepository.findAllByArenaIdWithDateFiltering(arenaId, fromDateTime, toDateTime);
    }

    public boolean checkTimeTableCollision(Long arenaId, Timestamp fromDateTime, Timestamp toDateTime) {
        return reservationRepository.timeCollisionExist(arenaId, fromDateTime, toDateTime) > 0;
    }

    public ReservationRecord findById(Long recordId) {
        return reservationRepository.findById(recordId)
                .orElseThrow(
                        () -> new ResourceNotFoundException("Reservation record with specified ID ("
                                + recordId + ") was not found")
                );
    }

    public void deleteRecord(Long recordId) {
        reservationRepository.deleteById(recordId);
    }

    public void deleteAll() {
        reservationRepository.deleteAll();
    }
}
