package cz.muni.fi.icehockeymanager.arenareservation.rest;

import com.fasterxml.jackson.databind.ObjectMapper;
import cz.muni.fi.icehockeymanager.arenareservation.api.create.CreateContactPersonDto;
import cz.muni.fi.icehockeymanager.arenareservation.api.view.ContactPersonDetailDto;
import cz.muni.fi.icehockeymanager.arenareservation.facade.ContactPersonFacade;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;

import java.nio.charset.StandardCharsets;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.is;
import static org.mockito.ArgumentMatchers.any;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@WebMvcTest(ContactPersonRestController.class)
public class ContactPersonRestControllerUnitTests {

    private static final Logger log = LoggerFactory.getLogger(ContactPersonRestControllerUnitTests.class);
    @Autowired
    private MockMvc mockMvc;
    @MockBean
    private ContactPersonFacade mockContactPersonFacade;
    @Autowired
    private ObjectMapper objectMapper;

    @Test
    public void testAddNewContactPerson() throws Exception {
        log.debug("Performing Unit Test for registering new ContactPerson");

        CreateContactPersonDto createContactPersonDto = new CreateContactPersonDto();
        createContactPersonDto.setName("John");
        createContactPersonDto.setSurname("Wick");
        createContactPersonDto.setEmail("john.wick123@example.mail");
        createContactPersonDto.setTelephoneNumber("+123 123 456 789");

        ContactPersonDetailDto contactPersonDetailDto = new ContactPersonDetailDto();
        contactPersonDetailDto.setId(1L);
        contactPersonDetailDto.setName(createContactPersonDto.getName());
        contactPersonDetailDto.setSurname(createContactPersonDto.getSurname());
        contactPersonDetailDto.setEmail(createContactPersonDto.getEmail());
        contactPersonDetailDto.setTelephoneNumber(createContactPersonDto.getTelephoneNumber());

        Mockito.when(mockContactPersonFacade.createContactPerson(any(CreateContactPersonDto.class)))
                .thenReturn(contactPersonDetailDto);

        String result = mockMvc.perform(post("/people")
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(objectMapper.writeValueAsString(createContactPersonDto)))
                .andExpect(status().isCreated())
                .andReturn().getResponse().getContentAsString(StandardCharsets.UTF_8);

        assertThat(result.equals(objectMapper.writeValueAsString(contactPersonDetailDto)), is(true));
    }

    @Test
    public void testDeleteArena() throws  Exception {
        log.debug("Performing Unit Test for deleting Contact Person based on Id");

        mockMvc.perform(delete("/people/1"))
                .andExpect(status().isNoContent());
        Mockito.verify(mockContactPersonFacade).deleteContactPerson(1L);
    }

    @Test
    public void testServerSideError() throws Exception {
        log.debug("Performing Unit Test for Server-side error");

        Mockito.when(mockContactPersonFacade.createContactPerson(any()))
                .thenThrow(new NullPointerException());

        mockMvc.perform(post("/people")
                .contentType(MediaType.APPLICATION_JSON)
                .content(objectMapper.writeValueAsString(new CreateContactPersonDto()))
        ).andExpect(status().is5xxServerError());
    }

    @Test
    public void testWrongUrl() throws Exception {
        log.debug("Performing Unit Test for incorrect URL");
        mockMvc.perform(post("/personas")
                .contentType(MediaType.APPLICATION_JSON)
                .content(objectMapper.writeValueAsString(new CreateContactPersonDto()))
        ).andExpect(status().is4xxClientError());
    }

    @Test
    public void testNonSupportedHttpMethodPut() throws Exception {
        log.debug("Performing Unit Test for not supported HTTP method");
        mockMvc.perform(put("/arenas/1")
                .contentType(MediaType.APPLICATION_JSON)
                .content(objectMapper.writeValueAsString(new ContactPersonDetailDto()))
        ).andExpect(status().is4xxClientError());
    }
}
