package cz.muni.fi.icehockeymanager.arenareservation.service;

import cz.muni.fi.icehockeymanager.arenareservation.data.domain.Address;
import cz.muni.fi.icehockeymanager.arenareservation.data.domain.Arena;
import cz.muni.fi.icehockeymanager.arenareservation.data.repository.ArenaRepository;
import cz.muni.fi.icehockeymanager.arenareservation.exceptions.ResourceNotFoundException;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;

import java.util.*;

import static org.mockito.ArgumentMatchers.any;

@SpringBootTest
public class ArenaServiceTests {
    @Autowired
    private ArenaService arenaService;

    @MockBean
    private ArenaRepository arenaRepository;

    private static List<Arena> arenas = new ArrayList<>();

    @BeforeAll
    public static void initBeforeTests() {
        Arena arenaOne = new Arena(1L, new Address(), "O2 Brno Arena",
                2023, 10200, 12343, new HashSet<>());
        Address arenaOneAddress = new Address(1L, "Botanická", "10/20",
                "Brno", "Moravia", "CZE", arenaOne);
        arenaOne.setAddress(arenaOneAddress);

        Arena arenaTwo = new Arena(2L, new Address(), "Mercedes-Benz Arena",
                2015, 8400, 12343, new HashSet<>());
        Address arenaTwoAddress = new Address(2L, "Benz", "100A/23II",
                "Berlin", "Berlin", "DEU", arenaTwo);
        arenaTwo.setAddress(arenaTwoAddress);


        Arena arenaThree = new Arena(3L, new Address(), "Helsinki Halli",
                1997, 6800, 12343, new HashSet<>());
        Address arenaThreeAddress = new Address(3L, "Areenankuja", "1",
                "Helsinki", "Helsinki", "FIN", arenaThree);
        arenaThree.setAddress(arenaThreeAddress);

        arenas.add(arenaOne);
        arenas.add(arenaTwo);
        arenas.add(arenaThree);
    }

    @Test
    public void testGetNonEmptyListOfArenas() {
        Mockito.when(arenaRepository.findAll()).thenReturn(arenas);
        List<Arena> arenaList = arenaService.getAllArenas();
        assert Objects.equals(arenaList.size(), arenas.size());

        int index = 0;

        for (Arena arena : arenaList) {
            assert arena.equals(arenas.get(index));
            index++;
        }
    }

    @Test
    public void TestGetEmptyListOfArenas() {
        Mockito.when(arenaRepository.findAll()).thenReturn(new ArrayList<>());
        List<Arena> arenaList = arenaService.getAllArenas();

        assert arenaList != null;
        assert Objects.equals(arenaList.size(), 0);
    }

    @Test
    public void testGetArenaWithExistingId() {
        Mockito.when(arenaRepository.findById(any(Long.class)))
                .thenReturn(Optional.ofNullable(arenas.get(0)));
        Arena arena = arenaService.findArenaById(1L);
        assert arena.equals(arenas.get(0));
    }

    @Test
    public void testGetArenaWithNonExistingId() {
        Mockito.when(arenaRepository.findById(any(Long.class)))
                .thenReturn(Optional.empty());

        try {
            arenaService.findArenaById(123L);
            assert false;
        }
        catch (ResourceNotFoundException resourceNotFoundException) {
            assert true;
        }
    }

    @Test
    public void testSaveArena() {
        Mockito.when(arenaRepository.save(any(Arena.class)))
                .thenReturn(arenas.get(0));

        Arena arena = arenaService.saveArena(arenas.get(0));
        assert arena.equals(arenas.get(0));
    }

    @Test
    public void testDeleteArena() {
        List<Long> ids = Arrays.asList(123L, 234L, 345L, 456L, 567L, 678L, 789L);

        for(Long id : ids) {
            arenaService.deleteArena(id);
            Mockito.verify(arenaRepository).deleteById(id);
        }
    }

    @Test
    public void testDeleteAllArenas() {
        arenaService.deleteAll();
        Mockito.verify(arenaRepository).deleteAll();
    }
}
