package cz.muni.fi.icehockeymanager.arenareservation.service;

import cz.muni.fi.icehockeymanager.arenareservation.data.domain.ContactPerson;
import cz.muni.fi.icehockeymanager.arenareservation.data.repository.ContactPersonRepository;
import cz.muni.fi.icehockeymanager.arenareservation.exceptions.ResourceNotFoundException;
import jakarta.transaction.Transactional;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;

import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Optional;

import static org.mockito.ArgumentMatchers.any;

@SpringBootTest
public class ContactPersonServiceTests {
    @Autowired
    ContactPersonService contactPersonService;

    @MockBean
    ContactPersonRepository contactPersonRepository;

    private static List<ContactPerson> contactPersonList = Arrays.asList(
            new ContactPerson(1L, "John", "Wick", "john.wick123@example.com", "+1234252", new HashSet<>()),
            new ContactPerson(2L, "Rick", "Astley", "rick1919@example.com", "+09412 412", new HashSet<>()),
            new ContactPerson(3L, "Jack", "Sparrow", "jack.sparrow@example.com", "+213 53234 432", new HashSet<>()),
            new ContactPerson(4L, "Real", "Person", "real.person@example.com", "+123 3123 321", new HashSet<>())
    );

    @Test
    public void testGetPersonWithSpecificId() {
        Mockito.when(contactPersonRepository.findById(any(Long.class)))
                .thenReturn(Optional.ofNullable(contactPersonList.get(1)));

        ContactPerson contactPerson = contactPersonService.findById(1L);
        assert contactPerson.equals(contactPersonList.get(1));
    }

    @Test
    public void testGetPersonWithNonExistingId() {
        Mockito.when(contactPersonRepository.findById(any(Long.class)))
                .thenReturn(Optional.empty());

        try {
            contactPersonService.findById(123L);
            assert false;
        }
        catch (ResourceNotFoundException resourceNotFoundException) {
            assert true;
        }
    }

    @Test
    public void testCreationOfPerson() {
        Mockito.when(contactPersonRepository.save(any(ContactPerson.class)))
                .thenReturn(contactPersonList.get(0));
        assert contactPersonService.save(contactPersonList.get(0)).equals(contactPersonList.get(0));
    }

    @Test
    public void testDeleteArena() {
        List<Long> ids = Arrays.asList(123L, 234L, 345L, 456L, 567L, 678L, 789L);

        for(Long id : ids) {
            contactPersonService.deleteById(id);
            Mockito.verify(contactPersonRepository).deleteById(id);
        }
    }

    @Test
    public void testDeleteAllArenas() {
        contactPersonService.deleteAll();
        Mockito.verify(contactPersonRepository).deleteAll();
    }
}
