package cz.muni.fi.icehockeymanager.discussionforum.data.domain;


import jakarta.persistence.*;
import jakarta.validation.constraints.NotNull;
import jakarta.validation.constraints.PastOrPresent;
import lombok.*;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

@Getter
@Setter
@NoArgsConstructor
@AllArgsConstructor
@Entity
@Table(name = "comment")
public class Comment implements Serializable {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    @NotNull
    @Column(length = 255)
    private String content;

    @PastOrPresent
    private Date createdAt;

    @PastOrPresent
    private Date lastEditedAt;

    @NotNull
    private Long authorId;

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "thread_id")
    private Thread thread;

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "parent_id")
    private Comment parentComment;

    @OneToMany(cascade = CascadeType.ALL, mappedBy = "parentComment", orphanRemoval = true)
    private List<Comment> replies;

    @Override
    public String toString() {
        return "Comment{" +
                "id=" + id +
                ", content='" + content + '\'' +
                ", createdAt=" + createdAt +
                ", lastEditedAt=" + lastEditedAt +
                ", authorId=" + authorId +
                ", replies=" + replies +
                '}';
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Comment comment = (Comment) o;
        return getId().equals(comment.getId()) &&
                getContent().equals(comment.getContent()) &&
                getCreatedAt().equals(comment.getCreatedAt()) &&
                getLastEditedAt().equals(comment.getLastEditedAt()) &&
                getAuthorId().equals(comment.getAuthorId()) &&
                getReplies().equals(comment.getReplies());
    }

    @Override
    public int hashCode() {
        return Objects.hash(getId(), getContent(), getCreatedAt(), getLastEditedAt(), getAuthorId(), getReplies());
    }
}