package cz.muni.fi.icehockeymanager.discussionforum.data.domain;

import jakarta.persistence.*;
import jakarta.validation.constraints.NotNull;
import jakarta.validation.constraints.PastOrPresent;
import lombok.*;

import java.io.Serializable;
import java.util.Date;
import java.util.List;
import java.util.Objects;

@Getter
@Setter
@NoArgsConstructor
@AllArgsConstructor
@Entity
@Table(name = "thread")
public class Thread implements Serializable {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    @NotNull
    @Column(length = 80)
    private String name;

    @NotNull
    @Column(length = 255)
    private String description;

    @PastOrPresent
    private Date createdAt;

    @PastOrPresent
    private Date lastEditedAt;

    @NotNull
    private Long authorId;

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "category_id")
    private Category category;

    @OneToMany(mappedBy = "thread", cascade = CascadeType.ALL, orphanRemoval = true)
    private List<Comment> comments;

    @Override
    public String toString() {
        return "Thread{" +
                "id=" + id +
                ", name='" + name + '\'' +
                ", description='" + description + '\'' +
                ", createdAt=" + createdAt +
                ", lastEditedAt=" + lastEditedAt +
                ", authorId=" + authorId +
                ", comments=" + comments +
                '}';
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Thread thread = (Thread) o;
        return getId().equals(thread.getId()) &&
                getName().equals(thread.getName()) &&
                getDescription().equals(thread.getDescription()) &&
                getCreatedAt().equals(thread.getCreatedAt()) &&
                getLastEditedAt().equals(thread.getLastEditedAt()) &&
                getAuthorId().equals(thread.getAuthorId()) &&
                getComments().equals(thread.getComments());
    }

    @Override
    public int hashCode() {
        return Objects.hash(getId(), getName(), getDescription(), getCreatedAt(), getLastEditedAt(), getAuthorId(), getComments());
    }
}
