package cz.muni.fi.icehockeymanager.discussionforum.facade;

import cz.muni.fi.icehockeymanager.discussionforum.api.dto.create.CreateCategoryDto;
import cz.muni.fi.icehockeymanager.discussionforum.api.dto.view.CategoryDto;
import cz.muni.fi.icehockeymanager.discussionforum.data.domain.Category;
import cz.muni.fi.icehockeymanager.discussionforum.mappers.CategoryMapper;
import cz.muni.fi.icehockeymanager.discussionforum.service.CategoryService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
@Transactional
public class CategoryFacade  {
    private final CategoryService categoryService;
    private final CategoryMapper categoryMapper;

    @Autowired
    public CategoryFacade(CategoryService categoryService, CategoryMapper categoryMapper) {
        this.categoryService = categoryService;
        this.categoryMapper = categoryMapper;
    }

    public List<CategoryDto> getAllCategories() {
        return categoryMapper.mapCategoryEntityListToCategoryDtoList(categoryService.getAllCategories());
    }

    public CategoryDto getCategoryById(Long categoryId) {
        return categoryMapper.mapCategoryEntityToCategoryDto(categoryService.getCategoryById(categoryId));
    }

    public CategoryDto createCategory(CreateCategoryDto createCategoryDto) {
        return categoryMapper.mapCategoryEntityToCategoryDto(
                categoryService.createCategory(
                        categoryMapper.mapCreateCategoryDtoToCategoryEntity(createCategoryDto)
                )
        );
    }

    public void deleteCategory(Long categoryId) {
        categoryService.deleteCategory(categoryId);
    }
}
