package cz.muni.fi.icehockeymanager.discussionforum.facade;

import cz.muni.fi.icehockeymanager.discussionforum.api.dto.create.CreateCommentDto;
import cz.muni.fi.icehockeymanager.discussionforum.api.dto.view.CommentDto;
import cz.muni.fi.icehockeymanager.discussionforum.data.domain.Comment;
import cz.muni.fi.icehockeymanager.discussionforum.mappers.CommentMapper;
import cz.muni.fi.icehockeymanager.discussionforum.service.CommentService;
import cz.muni.fi.icehockeymanager.discussionforum.service.ThreadService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
@Transactional
public class CommentFacade {
    private final CommentService commentService;
    private final ThreadService threadService;
    private final CommentMapper commentMapper;

    @Autowired
    public CommentFacade(CommentService commentService, ThreadService threadService, CommentMapper commentMapper) {
        this.commentService = commentService;
        this.threadService = threadService;
        this.commentMapper = commentMapper;
    }

    public List<CommentDto> getAllComments() {
        return commentMapper.mapCommentEntityListToCommentDtoList(
                commentService.getAllComments()
        );
    }

    public CommentDto getCommentById(Long commentId) {
        return commentMapper.mapCommentEntityToCommentDto(
                commentService.getCommentById(commentId)
        );
    }

    public CommentDto createRootComment(Long authorId, CreateCommentDto createCommentDto) {
        Comment comment = commentMapper.mapCreateCommentDtoToCommentEntity(createCommentDto);
        comment.setAuthorId(authorId);
        comment.setThread(threadService.getThreadById(createCommentDto.getThreadId()));

        return commentMapper.mapCommentEntityToCommentDto(
                commentService.createComment(comment)
        );
    }

    public CommentDto createReplyComment(Long authorId, CreateCommentDto createCommentDto) {
        Comment comment = commentMapper.mapCreateCommentDtoToCommentEntity(createCommentDto);
        comment.setAuthorId(authorId);
        comment.setThread(threadService.getThreadById(createCommentDto.getThreadId()));
        comment.setParentComment(commentService.getCommentById(createCommentDto.getParentId()));

        return commentMapper.mapCommentEntityToCommentDto(
                commentService.createComment(comment)
        );
    }

    public void deleteComment(Long commentId) {
        commentService.deleteComment(commentId);
    }
}
