package cz.muni.fi.icehockeymanager.discussionforum.facade;

import cz.muni.fi.icehockeymanager.discussionforum.api.dto.create.CreateThreadDto;
import cz.muni.fi.icehockeymanager.discussionforum.api.dto.view.ThreadDto;
import cz.muni.fi.icehockeymanager.discussionforum.data.domain.Thread;
import cz.muni.fi.icehockeymanager.discussionforum.mappers.ThreadMapper;
import cz.muni.fi.icehockeymanager.discussionforum.service.CategoryService;
import cz.muni.fi.icehockeymanager.discussionforum.service.ThreadService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
@Transactional
public class ThreadFacade {
    private final ThreadService threadService;
    private final CategoryService categoryService;
    private final ThreadMapper threadMapper;

    @Autowired
    public ThreadFacade(ThreadService threadService, CategoryService categoryService, ThreadMapper threadMapper) {
        this.threadService = threadService;
        this.categoryService = categoryService;
        this.threadMapper = threadMapper;
    }

    public List<ThreadDto> getAllThreads() {
        return threadMapper.mapThreadEntityListToThreadDtoList(
                threadService.getAllThreads()
        );
    }

    public ThreadDto getThreadById(Long threadId) {
        return threadMapper.mapThreadEntityToThreadDto(
                threadService.getThreadById(threadId)
        );
    }

    public ThreadDto createThread(Long authorId, CreateThreadDto createThreadDto) {
        Thread thread = threadMapper.mapCreateThreadDtoToThreadEntity(createThreadDto);
        thread.setAuthorId(authorId);
        thread.setCategory(categoryService.getCategoryById(createThreadDto.getCategoryId()));

        return threadMapper.mapThreadEntityToThreadDto(
                threadService.createThread(thread)
        );
    }

    public void deleteThread(Long threadId) {
        threadService.deleteThread(threadId);
    }
}
