package cz.muni.fi.icehockeymanager.discussionforum.rest;

import cz.muni.fi.icehockeymanager.discussionforum.api.dto.create.CreateCategoryDto;
import cz.muni.fi.icehockeymanager.discussionforum.api.dto.view.CategoryDto;
import cz.muni.fi.icehockeymanager.discussionforum.facade.CategoryFacade;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.security.oauth2.jwt.Jwt;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/categories")
public class CategoryRestController {
    private final CategoryFacade categoryFacade;

    @Autowired
    public CategoryRestController(CategoryFacade categoryFacade) {
        this.categoryFacade = categoryFacade;
    }

    @Operation(summary = "List all categories")
    @GetMapping
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<List<CategoryDto>> listCategories() {
        return ResponseEntity.ok(categoryFacade.getAllCategories());
    }

    @Operation(summary = "Get details about a specific category")
    @GetMapping("/{categoryId}")
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<CategoryDto> getCategory(@PathVariable("categoryId") Long categoryId) {
        return ResponseEntity.ok(categoryFacade.getCategoryById(categoryId));
    }

    @Operation(
        summary = "Create a new category",
        security = @SecurityRequirement(name = "Bearer")
    )
    @PostMapping
    @ResponseStatus(HttpStatus.CREATED)
    public ResponseEntity<CategoryDto> createCategory(@RequestBody CreateCategoryDto createCategoryDto) {
        return new ResponseEntity<>(categoryFacade.createCategory(createCategoryDto), HttpStatus.CREATED);
    }

    @Operation(
        summary = "Delete a category",
        security = @SecurityRequirement(name = "Bearer")
    )
    @DeleteMapping("/{categoryId}")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    public ResponseEntity<Void> deleteCategory(@PathVariable("categoryId") Long categoryId) {
        categoryFacade.deleteCategory(categoryId);
        return ResponseEntity.noContent().build();
    }
}

