package cz.muni.fi.icehockeymanager.discussionforum.rest;

import cz.muni.fi.icehockeymanager.discussionforum.api.dto.create.CreateThreadDto;
import cz.muni.fi.icehockeymanager.discussionforum.api.dto.view.ThreadDto;
import cz.muni.fi.icehockeymanager.discussionforum.api.dto.view.UserDto;
import cz.muni.fi.icehockeymanager.discussionforum.facade.ThreadFacade;
import cz.muni.fi.icehockeymanager.discussionforum.service.UserInfoService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.security.oauth2.jwt.Jwt;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/threads")
public class ThreadRestController {
    private final ThreadFacade threadFacade;
    private final UserInfoService userInfoService;

    @Autowired
    public ThreadRestController(ThreadFacade threadFacade, UserInfoService userInfoService) {
        this.threadFacade = threadFacade;
        this.userInfoService = userInfoService;
    }

    @Operation(summary = "List all discussion threads")
    @GetMapping
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<List<ThreadDto>> listThreads() {
        return ResponseEntity.ok(threadFacade.getAllThreads());
    }

    @Operation(summary = "Get details about a specific discussion thread")
    @GetMapping("/{threadId}")
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<ThreadDto> getThread(@PathVariable("threadId") Long threadId) {
        return ResponseEntity.ok(threadFacade.getThreadById(threadId));
    }

    @Operation(
        summary = "Create a new discussion thread",
        security = @SecurityRequirement(name = "Bearer")
    )
    @PostMapping
    @ResponseStatus(HttpStatus.CREATED)
    public ResponseEntity<ThreadDto> createThread(@AuthenticationPrincipal Jwt principal, @RequestBody CreateThreadDto createThreadDto) {
        Long authorId = 0L;
        if (principal != null) {
            UserDto userInfo = userInfoService.getUserInfo(principal);
            authorId = userInfo.getId();
        }
        return new ResponseEntity<>(threadFacade.createThread(authorId, createThreadDto), HttpStatus.CREATED);
    }

    @Operation(
        summary = "Delete a discussion thread",
        security = @SecurityRequirement(name = "Bearer")
    )
    @DeleteMapping("/{threadId}")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    public ResponseEntity<Void> deleteThread(@PathVariable("threadId") Long threadId) {
        threadFacade.deleteThread(threadId);
        return ResponseEntity.noContent().build();
    }
}
