package cz.muni.fi.icehockeymanager.discussionforum.service;

import cz.muni.fi.icehockeymanager.discussionforum.data.domain.Comment;
import cz.muni.fi.icehockeymanager.discussionforum.data.repository.CommentRepository;
import cz.muni.fi.icehockeymanager.discussionforum.exceptions.ResourceNotFoundException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

@Service
@Transactional
public class CommentService {
    private final CommentRepository commentRepository;

    @Autowired
    public CommentService(CommentRepository commentRepository) {
        this.commentRepository = commentRepository;
    }

    public Comment getCommentById(Long id) {
        return commentRepository.findById(id)
                .orElseThrow(
                        () -> new ResourceNotFoundException("Comment with specified ID (" + id + ") was not found")
                );
    }

    public List<Comment> getAllComments() {
        return commentRepository.findAll();
    }

    public Comment createComment(Comment comment) {
        comment.setCreatedAt(new Date());
        comment.setLastEditedAt(new Date());
        return commentRepository.save(comment);
    }

    public void deleteComment(Long id) {
        commentRepository.deleteById(id);
    }
}