package cz.muni.fi.icehockeymanager.discussionforum.service;

import cz.muni.fi.icehockeymanager.discussionforum.data.domain.Thread;
import cz.muni.fi.icehockeymanager.discussionforum.data.repository.ThreadRepository;
import cz.muni.fi.icehockeymanager.discussionforum.exceptions.ResourceNotFoundException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

@Service
@Transactional
public class ThreadService {
    private final ThreadRepository threadRepository;

    @Autowired
    public ThreadService(ThreadRepository threadRepository) {
        this.threadRepository = threadRepository;
    }

    public Thread getThreadById(Long id) {
        return threadRepository.findById(id)
                .orElseThrow(
                        () -> new ResourceNotFoundException("Thread with specified ID (" + id + ") was not found")
                );
    }

    public List<Thread> getAllThreads() {
        return threadRepository.findAll();
    }

    public Thread createThread(Thread thread) {
        thread.setCreatedAt(new Date());
        thread.setLastEditedAt(new Date());
        return threadRepository.save(thread);
    }

    public void deleteThread(Long id) {
        threadRepository.deleteById(id);
    }
}
