package cz.muni.fi.icehockeymanager.discussionforum.rest;

import cz.muni.fi.icehockeymanager.discussionforum.api.dto.create.CreateCategoryDto;
import cz.muni.fi.icehockeymanager.discussionforum.api.dto.view.CategoryDto;
import cz.muni.fi.icehockeymanager.discussionforum.facade.CategoryFacade;
import cz.muni.fi.icehockeymanager.discussionforum.service.UserInfoService;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;

import java.util.ArrayList;
import java.util.List;

import static org.mockito.ArgumentMatchers.any;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@WebMvcTest(controllers = CategoryRestController.class)
public class CategoryRestControllerTest {

    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private CategoryFacade categoryFacade;
    @MockBean
    private UserInfoService userInfoService;

    @Test
    public void testListCategories() throws Exception {
        List<CategoryDto> categories = new ArrayList<>();
        categories.add(getDummyCategory());

        Mockito.when(categoryFacade.getAllCategories()).thenReturn(categories);

        MvcResult response = mockMvc.perform(get("/categories"))
                .andExpect(status().isOk())
                .andReturn();

        Assertions.assertEquals(
                "[{\"id\":1,\"name\":\"Category 1\",\"threads\":[]}]",
                response.getResponse().getContentAsString()
        );
    }

    @Test
    public void testCreateCategory() throws Exception {

        Mockito.when(categoryFacade.createCategory(any(CreateCategoryDto.class)))
                .thenReturn(getDummyCategory());

        MvcResult response = mockMvc.perform(post("/categories")
                        .contentType("application/json")
                        .content("{\"name\":\"Category\"}"))
                .andExpect(status().isCreated())
                .andReturn();

        Assertions.assertEquals(
                "{\"id\":1,\"name\":\"Category 1\",\"threads\":[]}",
                response.getResponse().getContentAsString()
        );
    }

    @Test
    public void testDeleteCategory() throws Exception {
        mockMvc.perform(delete("/categories/1"))
                .andExpect(status().isNoContent());
        Mockito.verify(categoryFacade).deleteCategory(1L);
    }

    @Test
    public void testGetCategoryById() throws Exception {
        Mockito.when(categoryFacade.getCategoryById(1L))
                .thenReturn(getDummyCategory());

        MvcResult response = mockMvc.perform(get("/categories/1"))
                .andExpect(status().isOk())
                .andReturn();

        Assertions.assertEquals(
                "{\"id\":1,\"name\":\"Category 1\",\"threads\":[]}",
                response.getResponse().getContentAsString()
        );
    }

    private static CategoryDto getDummyCategory() {
        CategoryDto category = new CategoryDto();
        category.setId(1L);
        category.setName("Category 1");
        category.setThreads(new ArrayList<>());
        return category;
    }
}
