package cz.muni.fi.icehockeymanager.discussionforum.rest;

import cz.muni.fi.icehockeymanager.discussionforum.api.dto.create.CreateThreadDto;
import cz.muni.fi.icehockeymanager.discussionforum.api.dto.view.ThreadDto;
import cz.muni.fi.icehockeymanager.discussionforum.facade.ThreadFacade;
import cz.muni.fi.icehockeymanager.discussionforum.service.UserInfoService;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.TimeZone;

import static org.mockito.ArgumentMatchers.any;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@WebMvcTest(controllers = ThreadRestController.class)
public class ThreadRestControllerTest {

    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private ThreadFacade threadFacade;
    @MockBean
    private UserInfoService userInfoService;

    @Test
    public void testListThreads() throws Exception {
        List<ThreadDto> threads = new ArrayList<>();
        threads.add(getDummyThread());

        Mockito.when(threadFacade.getAllThreads()).thenReturn(threads);

        MvcResult response = mockMvc.perform(get("/threads"))
                .andExpect(status().isOk())
                .andReturn();

        Assertions.assertEquals(
                "[{\"id\":1,\"name\":\"Thread 1\",\"description\":\"description\"," +
                        "\"createdAt\":\"2023-03-26T10:00:00.000+00:00\"," +
                        "\"lastEditedAt\":\"2023-03-26T11:00:00.000+00:00\"," +
                        "\"authorId\":42,\"comments\":[]}]",
                response.getResponse().getContentAsString()
        );
    }

    @Test
    public void testCreateThread() throws Exception {
        ThreadDto thread = getDummyThread();

        Mockito.when(threadFacade.createThread(any(Long.class), any(CreateThreadDto.class))).thenReturn(thread);

        MvcResult response = mockMvc.perform(post("/threads")
                        .contentType("application/json")
                        .content("{\"name\":\"Category\"}"))
                .andExpect(status().isCreated())
                .andReturn();

        Assertions.assertEquals(
                "{\"id\":1,\"name\":\"Thread 1\",\"description\":\"description\"," +
                        "\"createdAt\":\"2023-03-26T10:00:00.000+00:00\"," +
                        "\"lastEditedAt\":\"2023-03-26T11:00:00.000+00:00\"," +
                        "\"authorId\":42,\"comments\":[]}",
                response.getResponse().getContentAsString()
        );
    }

    @Test
    public void testDeleteThread() throws Exception {
        mockMvc.perform(delete("/threads/1"))
                .andExpect(status().isNoContent());
        Mockito.verify(threadFacade).deleteThread(1L);
    }

    @Test
    public void testGetCategoryById() throws Exception {
        ThreadDto thread = getDummyThread();

        Mockito.when(threadFacade.getThreadById(1L)).thenReturn(thread);

        MvcResult response = mockMvc.perform(get("/threads/1"))
                .andExpect(status().isOk())
                .andReturn();

        Assertions.assertEquals(
                "{\"id\":1,\"name\":\"Thread 1\",\"description\":\"description\"," +
                        "\"createdAt\":\"2023-03-26T10:00:00.000+00:00\"," +
                        "\"lastEditedAt\":\"2023-03-26T11:00:00.000+00:00\"," +
                        "\"authorId\":42,\"comments\":[]}",
                response.getResponse().getContentAsString()
        );
    }

    private static ThreadDto getDummyThread() {
        ThreadDto thread = new ThreadDto();
        thread.setId(1L);
        thread.setName("Thread 1");
        thread.setDescription("description");
        thread.setAuthorId(42L);
        Calendar calendar = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
        calendar.clear();
        calendar.set(2023, Calendar.MARCH, 26, 10, 0, 0);
        thread.setCreatedAt(calendar.getTime());
        calendar.set(2023, Calendar.MARCH, 26, 11, 0, 0);
        thread.setLastEditedAt(calendar.getTime());
        thread.setComments(new ArrayList<>());
        return thread;
    }
}
