package cz.muni.fi.icehockeymanager.discussionforum.service;

import cz.muni.fi.icehockeymanager.discussionforum.data.domain.Category;
import cz.muni.fi.icehockeymanager.discussionforum.data.repository.CategoryRepository;
import cz.muni.fi.icehockeymanager.discussionforum.exceptions.ResourceNotFoundException;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;

import java.util.*;

import static org.mockito.ArgumentMatchers.any;

@SpringBootTest
public class CategoryServiceTest {

    @Autowired
    private CategoryService categoryService;

    @MockBean
    private CategoryRepository categoryRepository;

    private static final List<Category> categories = Arrays.asList(
            new Category(1L, "Ice Hockey Players", new ArrayList<>()),
            new Category(2L, "Ice Hockey Matches", new ArrayList<>()),
            new Category(3L, "NHL", new ArrayList<>()),
            new Category(4L, "Ice Hockey World Championship 2023", new ArrayList<>())
    );

    @Test
    public void testGetExistingCategoryWithId() {
        Mockito.when(categoryRepository.findById(1L))
                .thenReturn(Optional.ofNullable(categories.get(0)));
        Category category = categoryService.getCategoryById(1L);
        assert category.equals(categories.get(0));
    }

    @Test
    public void testGetNonExistingCategoryWithId() {
        Mockito.when(categoryRepository.findById(any(Long.class)))
                .thenReturn(Optional.empty());
        try {
            categoryService.getCategoryById(123L);
            assert false;
        } catch (ResourceNotFoundException resourceNotFoundException) {
            assert true;
        }
    }

    @Test
    public void testGetAllCategories() {
        Mockito.when(categoryRepository.findAll())
                .thenReturn(categories);
        List<Category> categoryList = categoryService.getAllCategories();
        assert Objects.equals(categoryList.size(), categories.size());

        int index = 0;
        for (Category category : categoryList) {
            assert category.equals(categories.get(index));
            index += 1;
        }
    }

    @Test
    public void testGetAllEmptyListOfCategories() {
        Mockito.when(categoryRepository.findAll())
                .thenReturn(new ArrayList<>());
        List<Category> categoryList = categoryService.getAllCategories();
        assert categoryList != null;
        assert categoryList.isEmpty();
    }

    @Test
    public void testCreateCategory() {
        Mockito.when(categoryRepository.save(any(Category.class)))
                .thenReturn(categories.get(1));
        Category newCategory = categoryService.createCategory(categories.get(1));
        assert newCategory.equals(categories.get(1));
        assert !newCategory.equals(categories.get(2));
    }

    @Test
    public void testDeleteCategory() {
        List<Long> ids = Arrays.asList(1L, 2L, 3L, 4L, 5L, 6L);
        for (Long id : ids) {
            categoryService.deleteCategory(id);
            Mockito.verify(categoryRepository).deleteById(id);
        }
    }
}
