package cz.muni.fi.icehockeymanager.core;


import cz.muni.fi.icehockeymanager.core.config.HockeyUserProperties;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Profile;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;

@SpringBootApplication
@EnableConfigurationProperties(HockeyUserProperties.class)
public class HockeyManagerApplication {

    public static void main( String[] args )
    {
        SpringApplication.run(HockeyManagerApplication.class, args);
    }

    /**
     * This filter chain is used when the application is running in the "secure" profile. This means that the
     * application requires authentication (login).
     */
    @Bean
    @Profile({"secure"})
    public SecurityFilterChain securityFilterChain(HttpSecurity httpSecurity) throws Exception {
        httpSecurity.csrf().disable().authorizeHttpRequests()
                .requestMatchers(new AntPathRequestMatcher("/actuator/prometheus")).permitAll()
                .requestMatchers(new AntPathRequestMatcher("/users/email/{email}", "GET")).permitAll()
                .requestMatchers(new AntPathRequestMatcher("/users/{id}", "DELETE")).hasAuthority("ADMIN")
                .requestMatchers(new AntPathRequestMatcher("/users", "POST")).hasAuthority("ADMIN")
                .requestMatchers(new AntPathRequestMatcher("/schedules/{id}", "DELETE")).hasAuthority("ADMIN")
                .requestMatchers(new AntPathRequestMatcher("/schedules", "POST")).hasAuthority("ADMIN")
                .requestMatchers(new AntPathRequestMatcher("/players/{id}", "DELETE")).hasAuthority("ADMIN")
                .requestMatchers(new AntPathRequestMatcher("/players", "POST")).hasAuthority("ADMIN")
                .requestMatchers(new AntPathRequestMatcher("/leagues/{id}", "DELETE")).hasAuthority("ADMIN")
                .requestMatchers(new AntPathRequestMatcher("/leagues", "POST")).hasAuthority("ADMIN")
                .requestMatchers(new AntPathRequestMatcher("/games/{id}", "DELETE")).hasAuthority("ADMIN")
                .requestMatchers(new AntPathRequestMatcher("/games", "POST")).hasAuthority("ADMIN")
                .requestMatchers(new AntPathRequestMatcher("/games/{id}", "PATCH")).hasAuthority("ADMIN")
                .anyRequest().authenticated()
                .and().oauth2Login().defaultSuccessUrl("/swagger-ui/index.html", true);
        return httpSecurity.build();
    }

    /**
     * This filter chain is used when the application is running in the "insecure" profile. This means that the
     * application does not require authentication (login). This is useful for running scenarios.
     */
    @Bean
    @Profile({"!secure"})
    public SecurityFilterChain securityFilterChainWithoutAuth(HttpSecurity httpSecurity) throws Exception {
        httpSecurity.csrf().disable().authorizeHttpRequests()
                .anyRequest().permitAll();
        return httpSecurity.build();
    }
}
