package cz.muni.fi.icehockeymanager.core.data.domain;

import cz.muni.fi.icehockeymanager.core.data.enums.GameStatus;
import jakarta.persistence.*;
import jakarta.validation.constraints.Min;
import jakarta.validation.constraints.Positive;
import lombok.*;

import java.io.Serializable;
import java.util.Date;

@Getter
@Setter
@NoArgsConstructor
@AllArgsConstructor
@ToString
@EqualsAndHashCode
@Entity
@Table(name = "game")
public class Game implements Serializable {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id")
    private Long id;
    @Column(name = "status", length = 11)
    @Enumerated(EnumType.STRING)
    private GameStatus status = GameStatus.NOT_STARTED;
    @ManyToOne
    @JoinColumn(name = "home_team_id")
    private Team homeTeam;
    @ManyToOne
    @JoinColumn(name = "away_team_id")
    private Team awayTeam;
    @Column(name = "home_team_goals")
    @Min(0)
    private Integer homeTeamGoals = 0;
    @Column(name = "away_team_goals")
    @Min(0)
    private Integer awayTeamGoals = 0;
    @Column(name = "date")
    private Date date;
    @ManyToOne
    @JoinColumn(name = "schedule_id")
    private Schedule schedule;

    public Game(Team homeTeam, Team awayTeam, Date date, Schedule schedule) {
        this.homeTeam = homeTeam;
        this.awayTeam = awayTeam;
        this.date = date;
        this.schedule = schedule;
    }
}
