package cz.muni.fi.icehockeymanager.core.data.domain;

import jakarta.persistence.*;
import lombok.*;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

@Getter
@Setter
@NoArgsConstructor
@AllArgsConstructor
@ToString
@EqualsAndHashCode
@Entity
@Table(name = "league")
public class League implements Serializable {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id")
    private Long id;
    @Column(name = "name", length = 100)
    private String name;
    @Column(name = "acronym", length = 10)
    private String acronym;
    @OneToMany(mappedBy = "league")
    private List<Enrollment> enrollments;
    @OneToMany(mappedBy = "league", cascade = {CascadeType.REMOVE}, orphanRemoval = true)
    private List<Schedule> schedules;

    public League(String name, String acronym) {
        this.name = name;
        this.acronym = acronym;
        this.enrollments = new ArrayList<>();
        this.schedules = new ArrayList<>();
    }

    public void addEnrollment(Enrollment enrollment) {
        enrollments.add(enrollment);
        enrollment.setLeague(this);
    }

    public void addSchedule(Schedule schedule) {
        schedules.add(schedule);
        schedule.setLeague(this);
    }
}
