package cz.muni.fi.icehockeymanager.core.data.domain;

import cz.muni.fi.icehockeymanager.core.data.enums.PlayerPosition;
import jakarta.persistence.*;
import lombok.*;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

@Getter
@Setter
@NoArgsConstructor
@AllArgsConstructor
@ToString
@EqualsAndHashCode
@Entity
@Table(name = "team")
public class Team implements Serializable {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id")
    private Long id;
    @Column(name = "name", length = 100)
    private String name;
    @ManyToOne
    @JoinColumn(name = "created_by")
    private User createdBy;
    @OneToMany(mappedBy = "team")
    private List<Player> players;
    @OneToMany(mappedBy = "team")
    private List<Enrollment> enrollments;

    public Team(String name, User createdBy) {
        this.name = name;
        this.createdBy = createdBy;
        this.players = new ArrayList<>();
        this.enrollments = new ArrayList<>();
    }

    public void addPlayer(Player player) {
        players.add(player);
        player.setTeam(this);
    }

    public void addEnrollment(Enrollment enrollment) {
        enrollments.add(enrollment);
        enrollment.setTeam(this);
    }

    public double getStrength() {
        return players.stream().mapToInt(Player::getStrength).sum() / (double) players.size();
    }

    public Boolean hasGoalkeeper() {
        return players.stream().anyMatch(player -> player.getPosition() == PlayerPosition.GOALTENDER);
    }
}
