package cz.muni.fi.icehockeymanager.core.facade;

import cz.muni.fi.icehockeymanager.core.api.create.CreateEnrollmentDto;
import cz.muni.fi.icehockeymanager.core.api.view.EnrollmentDto;
import cz.muni.fi.icehockeymanager.core.data.domain.Enrollment;
import cz.muni.fi.icehockeymanager.core.data.domain.League;
import cz.muni.fi.icehockeymanager.core.data.domain.Team;
import cz.muni.fi.icehockeymanager.core.mappers.EnrollmentMapper;
import cz.muni.fi.icehockeymanager.core.service.EnrollmentService;
import cz.muni.fi.icehockeymanager.core.service.LeagueService;
import cz.muni.fi.icehockeymanager.core.service.TeamService;
import jakarta.transaction.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;

@Service
@Transactional
public class EnrollmentFacade {

    private final EnrollmentService enrollmentService;
    private final TeamService teamService;
    private final LeagueService leagueService;
    private final EnrollmentMapper enrollmentMapper;

    @Autowired
    public EnrollmentFacade(EnrollmentService enrollmentService,
                            TeamService teamService,
                            LeagueService leagueService,
                            EnrollmentMapper enrollmentMapper)
    {
        this.enrollmentService = enrollmentService;
        this.teamService = teamService;
        this.leagueService = leagueService;
        this.enrollmentMapper = enrollmentMapper;
    }

    public List<EnrollmentDto> filterEnrollments(Optional<Long> teamId, Optional<Long> leagueId) {
        return enrollmentMapper.mapToListDto(enrollmentService.filterEnrollments(teamId, leagueId));
    }

    public EnrollmentDto createEnrollment(CreateEnrollmentDto createEnrollmentDto) {
        Team team = teamService.getTeamById(createEnrollmentDto.getTeamId());
        League league = leagueService.getLeagueById(createEnrollmentDto.getLeagueId());
        Enrollment enrollment = new Enrollment(team, league);

        team.addEnrollment(enrollment);
        league.addEnrollment(enrollment);
        return enrollmentMapper.mapToDto(enrollmentService.createEnrollment(enrollment));
    }

    public void deleteEnrollment(Long enrollmentId) {
        enrollmentService.deleteEnrollment(enrollmentId);
    }
}
