package cz.muni.fi.icehockeymanager.core.facade;

import cz.muni.fi.icehockeymanager.core.api.create.CreateGameDto;
import cz.muni.fi.icehockeymanager.core.api.view.GameDetailDto;
import cz.muni.fi.icehockeymanager.core.api.view.GameDto;
import cz.muni.fi.icehockeymanager.core.data.domain.Game;
import cz.muni.fi.icehockeymanager.core.data.domain.Team;
import cz.muni.fi.icehockeymanager.core.data.domain.Schedule;
import cz.muni.fi.icehockeymanager.core.mappers.GameMapper;
import cz.muni.fi.icehockeymanager.core.service.GameService;
import cz.muni.fi.icehockeymanager.core.service.TeamService;
import cz.muni.fi.icehockeymanager.core.service.ScheduleService;
import jakarta.transaction.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@Transactional
public class GameFacade {

    private final GameService gameService;
    private final ScheduleService scheduleService;
    private final TeamService teamService;
    private final GameMapper gameMapper;

    @Autowired
    public GameFacade(GameService gameService, ScheduleService scheduleService,
                      TeamService teamService, GameMapper gameMapper) {
        this.gameService = gameService;
        this.scheduleService = scheduleService;
        this.teamService = teamService;
        this.gameMapper = gameMapper;
    }

    public List<GameDto> getAllGames() {
        return gameMapper.mapToListDto(gameService.getAllGames());
    }

    public GameDetailDto createGame(CreateGameDto createGameDto) {
        Schedule schedule = scheduleService.getScheduleById(createGameDto.getScheduleId());
        Team homeTeam = teamService.getTeamById(createGameDto.getHomeTeamId());
        Team awayTeam = teamService.getTeamById(createGameDto.getAwayTeamId());

        Game game = new Game(homeTeam, awayTeam, createGameDto.getDate(), schedule);
        return gameMapper.mapToDetailDto(gameService.createGame(game));
    }

    public void deleteGame(Long gameId) {
        gameService.deleteGame(gameId);
    }

    public GameDetailDto getGameById(Long gameId) {
        return gameMapper.mapToDetailDto(gameService.getGameById(gameId));
    }

    public GameDetailDto startGame(Long gameId) {
        return gameMapper.mapToDetailDto(gameService.startGame(gameId));
    }
}
