package cz.muni.fi.icehockeymanager.core.facade;

import cz.muni.fi.icehockeymanager.core.api.create.CreateLeagueDto;
import cz.muni.fi.icehockeymanager.core.api.view.LeagueDetailDto;
import cz.muni.fi.icehockeymanager.core.api.view.LeagueDto;
import cz.muni.fi.icehockeymanager.core.data.domain.League;
import cz.muni.fi.icehockeymanager.core.mappers.LeagueMapper;
import cz.muni.fi.icehockeymanager.core.mappers.ScheduleMapper;
import cz.muni.fi.icehockeymanager.core.mappers.TeamMapper;
import cz.muni.fi.icehockeymanager.core.service.LeagueService;
import jakarta.transaction.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@Transactional
public class LeagueFacade {

    private final LeagueService leagueService;
    private final LeagueMapper leagueMapper;
    private final TeamMapper teamMapper;
    private final ScheduleMapper scheduleMapper;

    @Autowired
    public LeagueFacade(LeagueService leagueService, LeagueMapper leagueMapper, TeamMapper teamMapper,
                        ScheduleMapper scheduleMapper)
    {
        this.leagueService = leagueService;
        this.leagueMapper = leagueMapper;
        this.teamMapper = teamMapper;
        this.scheduleMapper = scheduleMapper;
    }

    public List<LeagueDto> getAllLeagues() {
        return leagueMapper.mapToListDto(leagueService.getAllLeagues());
    }

    public LeagueDetailDto getLeagueById(Long leagueId) {
        LeagueDetailDto leagueDetailDto = leagueMapper.mapToDetailDto(leagueService.getLeagueById(leagueId));
        leagueDetailDto.setTeams(teamMapper.mapToListDto(leagueService.getTeamsByLeagueId(leagueId)));
        leagueDetailDto.setSchedules(scheduleMapper.mapToListDto(leagueService.getSchedulesByLeagueId(leagueId)));

        return leagueDetailDto;
    }

    public LeagueDto createLeague(CreateLeagueDto createLeagueDto) {
        League league = new League(createLeagueDto.getName(), createLeagueDto.getAcronym());
        return leagueMapper.mapToDto(leagueService.createLeague(league));
    }

    public void deleteLeague(Long leagueId) {
        leagueService.deleteLeague(leagueId);
    }
}
