package cz.muni.fi.icehockeymanager.core.facade;

import cz.muni.fi.icehockeymanager.core.api.create.CreatePlayerDto;
import cz.muni.fi.icehockeymanager.core.api.view.PlayerDto;
import cz.muni.fi.icehockeymanager.core.data.domain.Player;
import cz.muni.fi.icehockeymanager.core.data.enums.PlayerPosition;
import cz.muni.fi.icehockeymanager.core.mappers.PlayerMapper;
import cz.muni.fi.icehockeymanager.core.service.PlayerService;
import jakarta.transaction.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@Transactional
public class PlayerFacade {

    private final PlayerService playerService;
    private final PlayerMapper playerMapper;

    @Autowired
    public PlayerFacade(PlayerService playerService, PlayerMapper playerMapper) {
        this.playerService = playerService;
        this.playerMapper = playerMapper;
    }

    public List<PlayerDto> getAllPlayers() {
        return playerMapper.mapToListDto(playerService.getAllPlayers());
    }

    public PlayerDto createPlayer(CreatePlayerDto createPlayerDto) {
        Player player = new Player(createPlayerDto.getFirstName(), createPlayerDto.getLastName(),
                createPlayerDto.getStrength(), createPlayerDto.getPosition());
        return playerMapper.mapToDto(playerService.createPlayer(player));
    }

    public void deletePlayer(Long playerId) {
        playerService.deletePlayer(playerId);
    }
}
