package cz.muni.fi.icehockeymanager.core.facade;

import cz.muni.fi.icehockeymanager.core.api.create.CreateScheduleDto;
import cz.muni.fi.icehockeymanager.core.api.view.ScheduleDetailDto;
import cz.muni.fi.icehockeymanager.core.api.view.ScheduleDto;
import cz.muni.fi.icehockeymanager.core.data.domain.League;
import cz.muni.fi.icehockeymanager.core.data.domain.Schedule;
import cz.muni.fi.icehockeymanager.core.mappers.ScheduleMapper;
import cz.muni.fi.icehockeymanager.core.service.LeagueService;
import cz.muni.fi.icehockeymanager.core.service.ScheduleService;
import jakarta.transaction.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@Transactional
public class ScheduleFacade {

    private final ScheduleService scheduleService;
    private final LeagueService leagueService;
    private final ScheduleMapper scheduleMapper;

    @Autowired
    public ScheduleFacade(ScheduleService scheduleService,
                          LeagueService leagueService,
                          ScheduleMapper scheduleMapper) {
        this.scheduleService = scheduleService;
        this.leagueService = leagueService;
        this.scheduleMapper = scheduleMapper;
    }

    public List<ScheduleDto> getAllSchedules() {
        return scheduleMapper.mapToListDto(scheduleService.getAllSchedules());
    }

    public ScheduleDetailDto createSchedule(CreateScheduleDto createScheduleDto) {
        League league = leagueService.getLeagueById(createScheduleDto.getLeagueId());
        Schedule schedule = new Schedule(createScheduleDto.getYear(), league);
        league.addSchedule(schedule);
        return scheduleMapper.mapToDetailDto(scheduleService.createSchedule(schedule));
    }

    public void deleteSchedule(Long id) {
        scheduleService.deleteSchedule(id);
    }

    public ScheduleDetailDto getScheduleById(Long id) {
        return scheduleMapper.mapToDetailDto(scheduleService.getScheduleById(id));
    }
}
