package cz.muni.fi.icehockeymanager.core.facade;

import cz.muni.fi.icehockeymanager.core.api.create.CreateTeamDto;
import cz.muni.fi.icehockeymanager.core.api.view.TeamDto;
import cz.muni.fi.icehockeymanager.core.api.view.TeamDetailDto;
import cz.muni.fi.icehockeymanager.core.data.domain.Player;
import cz.muni.fi.icehockeymanager.core.data.domain.Team;
import cz.muni.fi.icehockeymanager.core.data.domain.User;
import cz.muni.fi.icehockeymanager.core.mappers.TeamMapper;
import cz.muni.fi.icehockeymanager.core.service.PlayerService;
import cz.muni.fi.icehockeymanager.core.service.TeamService;
import cz.muni.fi.icehockeymanager.core.service.UserService;
import jakarta.transaction.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;

@Service
@Transactional
public class TeamFacade {

    private final TeamService teamService;
    private final UserService userService;
    private final PlayerService playerService;
    private final TeamMapper teamMapper;

    @Autowired
    public TeamFacade(TeamService teamService,
                      UserService userService,
                      PlayerService playerService,
                      TeamMapper teamMapper) {
        this.teamService = teamService;
        this.userService = userService;
        this.playerService = playerService;
        this.teamMapper = teamMapper;
    }


    public List<TeamDto> getAllTeams(Optional<Long> createdById) {
        return teamMapper.mapToListDto(teamService.getAllTeams(createdById));
    }

    public TeamDetailDto createTeam(CreateTeamDto createTeamDto) {
        User createdBy = userService.getUserById(createTeamDto.getCreatedById());
        Team team = new Team(createTeamDto.getName(), createdBy);
        return teamMapper.mapToDetailDto(teamService.createTeam(team));
    }

    public TeamDetailDto getTeamById(Long teamId) {
        return teamMapper.mapToDetailDto(teamService.getTeamById(teamId));
    }

    public void deleteTeam(Long teamId) {
        teamService.deleteTeam(teamId);
    }

    public TeamDetailDto addPlayer(Long teamId, Long playerId) {
        Player player = playerService.getPlayerById(playerId);
        return teamMapper.mapToDetailDto(teamService.addPlayer(teamId, player));
    }
}
