package cz.muni.fi.icehockeymanager.core.facade;

import cz.muni.fi.icehockeymanager.core.api.create.CreateUserDto;
import cz.muni.fi.icehockeymanager.core.api.view.UserDto;
import cz.muni.fi.icehockeymanager.core.data.domain.User;
import cz.muni.fi.icehockeymanager.core.mappers.UserMapper;
import cz.muni.fi.icehockeymanager.core.service.UserService;
import jakarta.transaction.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@Transactional
public class UserFacade {

    private final UserService userService;
    private final UserMapper userMapper;

    @Autowired
    public UserFacade(UserService userService, UserMapper userMapper) {
        this.userService = userService;
        this.userMapper = userMapper;
    }

    public List<UserDto> getAllUsers() {
        return userMapper.mapToListDto(userService.getAllUsers());
    }

    public UserDto registerUser(CreateUserDto createUserDto) {
        User user = new User(createUserDto.getFirstName(), createUserDto.getLastName(), createUserDto.getEmail());
        return userMapper.mapToDto(userService.createUser(user));
    }

    public void deleteUser(Long userId) {
        userService.deleteUser(userId);
    }

    public UserDto getUserById(Long userId) {
        return userMapper.mapToDto(userService.getUserById(userId));
    }

    public UserDto getUserByEmail(String email) {
        return userMapper.mapToDto(userService.getUserByEmail(email));
    }
}
