package cz.muni.fi.icehockeymanager.core.rest;

import cz.muni.fi.icehockeymanager.core.api.create.CreateEnrollmentDto;
import cz.muni.fi.icehockeymanager.core.api.view.EnrollmentDto;
import cz.muni.fi.icehockeymanager.core.facade.EnrollmentFacade;
import io.swagger.v3.oas.annotations.Operation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Optional;

@RestController
@RequestMapping("/enrollments")
public class EnrollmentRestController {

    private final EnrollmentFacade enrollmentFacade;

    @Autowired
    public EnrollmentRestController(EnrollmentFacade enrollmentFacade) {
        this.enrollmentFacade = enrollmentFacade;
    }

    @Operation(summary = "Filter and list enrollments")
    @GetMapping
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<List<EnrollmentDto>> listEnrollments(@RequestParam(value = "teamId") Optional<Long> teamId,
                                                               @RequestParam(value = "leagueId") Optional<Long> leagueId) {
        return ResponseEntity.ok(enrollmentFacade.filterEnrollments(teamId, leagueId));
    }

    @Operation(summary = "Enroll team to league")
    @PostMapping
    @ResponseStatus(HttpStatus.CREATED)
    public ResponseEntity<EnrollmentDto> createEnrollment(@RequestBody CreateEnrollmentDto createEnrollmentDto) {
        return new ResponseEntity<EnrollmentDto>(enrollmentFacade.createEnrollment(createEnrollmentDto), HttpStatus.CREATED);
    }

    @Operation(summary = "Exclude team from league")
    @DeleteMapping("/{enrollmentId}")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    public ResponseEntity<Void> deleteEnrollment(@PathVariable("enrollmentId") Long enrollmentId) {
        enrollmentFacade.deleteEnrollment(enrollmentId);
        return ResponseEntity.noContent().build();
    }
}
