package cz.muni.fi.icehockeymanager.core.rest;

import cz.muni.fi.icehockeymanager.core.api.create.CreateGameDto;
import cz.muni.fi.icehockeymanager.core.api.view.GameDetailDto;
import cz.muni.fi.icehockeymanager.core.api.view.GameDto;
import cz.muni.fi.icehockeymanager.core.facade.GameFacade;
import io.swagger.v3.oas.annotations.Operation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/games")
public class GameRestController {

    private final GameFacade gameFacade;

    @Autowired
    public GameRestController(GameFacade gameFacade) {
        this.gameFacade = gameFacade;
    }

    @Operation(summary = "List all games")
    @GetMapping
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<List<GameDto>> listGames() {
        return ResponseEntity.ok(gameFacade.getAllGames());
    }

    @Operation(summary = "Create game")
    @PostMapping
    @ResponseStatus(HttpStatus.CREATED)
    public ResponseEntity<GameDetailDto> createGame(@RequestBody CreateGameDto createGameDto) {
        return new ResponseEntity<GameDetailDto>(gameFacade.createGame(createGameDto), HttpStatus.CREATED);
    }

    @Operation(summary = "Delete game")
    @DeleteMapping("/{gameId}")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    public ResponseEntity<Void> deleteGame(@PathVariable("gameId") Long gameId) {
        gameFacade.deleteGame(gameId);
        return ResponseEntity.noContent().build();
    }

    @Operation(summary = "Get details of game")
    @GetMapping("/{gameId}")
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<GameDetailDto> getGame(@PathVariable("gameId") Long gameId) {
        return ResponseEntity.ok(gameFacade.getGameById(gameId));
    }

    @Operation(summary = "Start game")
    @PatchMapping("/{gameId}")
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<GameDetailDto> startGame(@PathVariable("gameId") Long gameId) {
        return ResponseEntity.ok(gameFacade.startGame(gameId));
    }
}
