package cz.muni.fi.icehockeymanager.core.rest;

import cz.muni.fi.icehockeymanager.core.api.create.CreateLeagueDto;
import cz.muni.fi.icehockeymanager.core.api.view.LeagueDetailDto;
import cz.muni.fi.icehockeymanager.core.api.view.LeagueDto;
import cz.muni.fi.icehockeymanager.core.facade.LeagueFacade;
import io.swagger.v3.oas.annotations.Operation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/leagues")
public class LeagueRestController {

    private final LeagueFacade leagueFacade;

    @Autowired
    public LeagueRestController(LeagueFacade leagueFacade) {
        this.leagueFacade = leagueFacade;
    }

    @Operation(summary = "List leagues")
    @GetMapping
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<List<LeagueDto>> listLeagues() {
        return ResponseEntity.ok(leagueFacade.getAllLeagues());
    }

    @Operation(summary = "Create league")
    @PostMapping
    @ResponseStatus(HttpStatus.CREATED)
    public ResponseEntity<LeagueDto> createLeague(@RequestBody CreateLeagueDto createLeagueDto) {
        return new ResponseEntity<LeagueDto>(leagueFacade.createLeague(createLeagueDto), HttpStatus.CREATED);
    }

    @Operation(summary = "Delete league")
    @DeleteMapping("/{leagueId}")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    public ResponseEntity<Void> deleteLeague(@PathVariable("leagueId") Long leagueId) {
        leagueFacade.deleteLeague(leagueId);
        return ResponseEntity.noContent().build();
    }

    @Operation(summary = "Get detailed information about league")
    @GetMapping("/{leagueId}")
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<LeagueDetailDto> getLeagueDetail(@PathVariable("leagueId") Long leagueId) {
        return ResponseEntity.ok(leagueFacade.getLeagueById(leagueId));
    }
}
