package cz.muni.fi.icehockeymanager.core.rest;

import cz.muni.fi.icehockeymanager.core.api.create.CreatePlayerDto;
import cz.muni.fi.icehockeymanager.core.api.view.PlayerDto;
import cz.muni.fi.icehockeymanager.core.facade.PlayerFacade;
import io.swagger.v3.oas.annotations.Operation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/players")
public class PlayerRestController {

    private final PlayerFacade playerFacade;

    @Autowired
    public PlayerRestController(PlayerFacade playerFacade) {
        this.playerFacade = playerFacade;
    }

    @Operation(summary = "List players")
    @GetMapping
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<List<PlayerDto>> listPlayers() {
        return ResponseEntity.ok(playerFacade.getAllPlayers());
    }

    @Operation(summary = "Create player")
    @PostMapping
    @ResponseStatus(HttpStatus.CREATED)
    public ResponseEntity<PlayerDto> createPlayer(@RequestBody CreatePlayerDto createPlayerDto) {
        return new ResponseEntity<PlayerDto>(playerFacade.createPlayer(createPlayerDto), HttpStatus.CREATED);
    }

    @Operation(summary = "Delete player")
    @DeleteMapping("/{playerId}")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    public ResponseEntity<Void> deletePlayer(@PathVariable("playerId") Long playerId) {
        playerFacade.deletePlayer(playerId);
        return ResponseEntity.noContent().build();
    }

}
