package cz.muni.fi.icehockeymanager.core.rest;

import cz.muni.fi.icehockeymanager.core.api.create.CreateScheduleDto;
import cz.muni.fi.icehockeymanager.core.api.view.ScheduleDetailDto;
import cz.muni.fi.icehockeymanager.core.api.view.ScheduleDto;
import cz.muni.fi.icehockeymanager.core.facade.ScheduleFacade;
import io.swagger.v3.oas.annotations.Operation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.http.HttpStatus;

import java.util.List;


@RestController
@RequestMapping("/schedules")
public class ScheduleRestController {

    private final ScheduleFacade scheduleFacade;

    @Autowired
    public ScheduleRestController(ScheduleFacade scheduleFacade) {
        this.scheduleFacade = scheduleFacade;
    }

    @Operation(summary = "List schedules")
    @GetMapping
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<List<ScheduleDto>> listSchedules() {
        return ResponseEntity.ok(scheduleFacade.getAllSchedules());
    }

    @Operation(summary = "Create schedule")
    @PostMapping
    @ResponseStatus(HttpStatus.CREATED)
    public ResponseEntity<ScheduleDetailDto> createSchedule(@RequestBody CreateScheduleDto createScheduleDto) {
        return new ResponseEntity<ScheduleDetailDto>(scheduleFacade.createSchedule(createScheduleDto), HttpStatus.CREATED);
    }

    @Operation(summary = "Delete schedule")
    @DeleteMapping("/{scheduleId}")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    public ResponseEntity<Void> deleteSchedule(@PathVariable Long scheduleId) {
        scheduleFacade.deleteSchedule(scheduleId);
        return ResponseEntity.noContent().build();
    }

    @Operation(summary = "Get schedule")
    @GetMapping("/{scheduleId}")
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<ScheduleDetailDto> getSchedule(@PathVariable Long scheduleId) {
        return ResponseEntity.ok(scheduleFacade.getScheduleById(scheduleId));
    }
}
