package cz.muni.fi.icehockeymanager.core.rest;

import cz.muni.fi.icehockeymanager.core.api.create.CreateTeamDto;
import cz.muni.fi.icehockeymanager.core.api.view.TeamDetailDto;
import cz.muni.fi.icehockeymanager.core.api.view.TeamDto;
import cz.muni.fi.icehockeymanager.core.facade.TeamFacade;
import io.swagger.v3.oas.annotations.Operation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Optional;

@RestController
@RequestMapping("/teams")
public class TeamRestController {

    private final TeamFacade teamFacade;

    @Autowired
    public TeamRestController(TeamFacade teamFacade) {
        this.teamFacade = teamFacade;
    }

    @Operation(summary = "List teams")
    @GetMapping
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<List<TeamDto>> listTeams(@RequestParam("createdById") Optional<Long> createdById) {
        return ResponseEntity.ok(teamFacade.getAllTeams(createdById));
    }

    @Operation(summary = "Create team")
    @PostMapping
    @ResponseStatus(HttpStatus.CREATED)
    public ResponseEntity<TeamDetailDto> createTeam(@RequestBody CreateTeamDto createTeamDto) {
        return new ResponseEntity<TeamDetailDto>(teamFacade.createTeam(createTeamDto), HttpStatus.CREATED);
    }

    @Operation(summary = "Get team by id")
    @GetMapping("/{teamId}")
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<TeamDetailDto> getTeam(@PathVariable("teamId") Long teamId) {
        return ResponseEntity.ok(teamFacade.getTeamById(teamId));
    }

    @Operation(summary = "Delete team")
    @DeleteMapping("/{teamId}")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    public ResponseEntity<Void> deleteTeam(@PathVariable("teamId") Long teamId) {
        teamFacade.deleteTeam(teamId);
        return ResponseEntity.noContent().build();
    }

    @Operation(summary = "Add player to team")
    @PatchMapping("/{teamId}/players/{playerId}")
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<TeamDetailDto> addPlayer(@PathVariable("teamId") Long teamId, @PathVariable("playerId") Long playerId) {
        return ResponseEntity.ok(teamFacade.addPlayer(teamId, playerId));
    }
}
