package cz.muni.fi.icehockeymanager.core.rest;

import cz.muni.fi.icehockeymanager.core.api.create.CreateUserDto;
import cz.muni.fi.icehockeymanager.core.api.view.UserDto;
import cz.muni.fi.icehockeymanager.core.facade.UserFacade;
import io.swagger.v3.oas.annotations.Operation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/users")
public class UserRestController {

    private final UserFacade userFacade;

    @Autowired
    public UserRestController(UserFacade userFacade) {
        this.userFacade = userFacade;
    }

    @Operation(summary = "List users")
    @GetMapping
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<List<UserDto>> listUsers() {
        return ResponseEntity.ok(userFacade.getAllUsers());
    }

    @Operation(summary = "Register user")
    @PostMapping
    @ResponseStatus(HttpStatus.CREATED)
    public ResponseEntity<UserDto> registerUser(@RequestBody CreateUserDto createUserDto) {
        return new ResponseEntity<UserDto>(userFacade.registerUser(createUserDto), HttpStatus.CREATED);
    }

    @Operation(summary = "Delete user")
    @DeleteMapping("/{userId}")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    public void deleteUser(@PathVariable Long userId) {
        userFacade.deleteUser(userId);
    }

    @Operation(summary = "Get user")
    @GetMapping("/{userId}")
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<UserDto> getUser(@PathVariable Long userId) {
        return ResponseEntity.ok(userFacade.getUserById(userId));
    }

    @Operation(summary = "Get user by email")
    @GetMapping("/email/{email}")
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<UserDto> getUserByEmail(@PathVariable String email) {
        return ResponseEntity.ok(userFacade.getUserByEmail(email));
    }
}
