package cz.muni.fi.icehockeymanager.core.rest.restexceptionhandling;

import cz.muni.fi.icehockeymanager.core.exceptions.ResourceNotFoundException;
import cz.muni.fi.icehockeymanager.core.exceptions.TeamCompositionException;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.context.request.WebRequest;
import org.springframework.web.method.annotation.MethodArgumentTypeMismatchException;
import org.springframework.web.servlet.mvc.method.annotation.ResponseEntityExceptionHandler;

@ControllerAdvice
public class IceHockeyManagerExceptionHandling extends ResponseEntityExceptionHandler {
    public IceHockeyManagerExceptionHandling() {
        super();
    }

    @ExceptionHandler({ MethodArgumentTypeMismatchException.class })
    public ResponseEntity<Object> handleMethodArgumentMismatch(final MethodArgumentTypeMismatchException exception,
                                                               final WebRequest request) {
        final String responseBody = "Client-Side Error: " + exception.getLocalizedMessage();
        return handleExceptionInternal(exception, responseBody,
                new HttpHeaders(), HttpStatus.BAD_REQUEST, request);
    }

    @ExceptionHandler({ TeamCompositionException.class })
    public ResponseEntity<Object> handleTeamCompositionException(final TeamCompositionException exception,
                                                                 final WebRequest request) {
        final String responseBody = "Client-Side Error: " + exception.getLocalizedMessage();
        return handleExceptionInternal(exception, responseBody,
                new HttpHeaders(), HttpStatus.BAD_REQUEST, request);
    }

    @ExceptionHandler({ ResourceNotFoundException.class })
    public ResponseEntity<Object> handleResourceNotFound(final ResourceNotFoundException exception, final WebRequest request) {
        final String responseBody = "Client-Side Error: " + exception.getLocalizedMessage();
        return handleExceptionInternal(exception, responseBody,
                new HttpHeaders(), HttpStatus.NOT_FOUND, request);
    }

    @ExceptionHandler({ NullPointerException.class, IllegalArgumentException.class, IllegalStateException.class })
    public ResponseEntity<Object> handleInternal(final RuntimeException exception, final WebRequest request) {
        final String responseBody = "Server-Side Error: " + exception.getLocalizedMessage();
        return handleExceptionInternal(exception, responseBody,
                new HttpHeaders(), HttpStatus.INTERNAL_SERVER_ERROR, request);
    }
}
