package cz.muni.fi.icehockeymanager.core.service;

import cz.muni.fi.icehockeymanager.core.data.domain.Enrollment;
import cz.muni.fi.icehockeymanager.core.data.repository.EnrollmentRepository;
import cz.muni.fi.icehockeymanager.core.exceptions.ResourceNotFoundException;
import jakarta.transaction.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
@Transactional
public class EnrollmentService {

    private final EnrollmentRepository enrollmentRepository;

    @Autowired
    public EnrollmentService(EnrollmentRepository enrollmentRepository) {
        this.enrollmentRepository = enrollmentRepository;
    }

    public List<Enrollment> filterEnrollments(Optional<Long> teamId, Optional<Long> leagueId) {
        return enrollmentRepository.findAll()
                .stream()
                .filter(enrollment ->
                        (!teamId.isPresent() || enrollment.getTeam().getId().equals(teamId.get())) &&
                                (!leagueId.isPresent() || enrollment.getLeague().getId().equals(leagueId.get())))
                .collect(Collectors.toList());
    }

    public Enrollment createEnrollment(Enrollment enrollment) {
        return enrollmentRepository.save(enrollment);
    }

    public void deleteEnrollment(Long enrollmentId) {
        if (enrollmentRepository.deleteEnrollmentById(enrollmentId) != 1) {
            throw new ResourceNotFoundException("Enrollment with id " + enrollmentId + " does not exist");
        }
    }
}
