package cz.muni.fi.icehockeymanager.core.service;

import cz.muni.fi.icehockeymanager.core.config.HockeyUserProperties;
import cz.muni.fi.icehockeymanager.core.data.domain.User;
import cz.muni.fi.icehockeymanager.core.exceptions.LoginException;
import cz.muni.fi.icehockeymanager.core.service.utils.HelperAuthService;
import jakarta.transaction.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.oauth2.client.userinfo.DefaultOAuth2UserService;
import org.springframework.security.oauth2.client.userinfo.OAuth2UserRequest;
import org.springframework.security.oauth2.core.user.DefaultOAuth2User;
import org.springframework.security.oauth2.core.user.OAuth2User;
import org.springframework.stereotype.Service;

import java.util.Set;

@Service
public class HockeyOAuth2UserService extends DefaultOAuth2UserService {

    private final HelperAuthService<OAuth2User> helperAuthService;

    @Autowired
    public HockeyOAuth2UserService(UserService userService, HockeyUserProperties hockeyUserProperties) {
        this.helperAuthService = new HelperAuthService<>(userService, hockeyUserProperties);
    }

    @Override
    @Transactional
    public OAuth2User loadUser(OAuth2UserRequest userRequest) {
        OAuth2User oAuth2User = super.loadUser(userRequest);
        String fullName = oAuth2User.getAttribute("name");
        String email = (String) oAuth2User.getAttributes().get("email");

        if (fullName == null || email == null) {
            throw new LoginException("Could not get user info from Github!" +
                    " Make sure you have public name and email set.");
        }

        String firstName = fullName.split(" ")[0];
        String lastName = fullName.split(" ")[1];

        User user = helperAuthService.getUser(firstName, lastName, email);
        Set<GrantedAuthority> authorities = helperAuthService.getAuthorities(user, oAuth2User);
        return new DefaultOAuth2User(
                authorities,
                oAuth2User.getAttributes(),
                "email"
        );
    }

}
