package cz.muni.fi.icehockeymanager.core.service;

import cz.muni.fi.icehockeymanager.core.config.HockeyUserProperties;
import cz.muni.fi.icehockeymanager.core.data.domain.User;
import cz.muni.fi.icehockeymanager.core.exceptions.LoginException;
import cz.muni.fi.icehockeymanager.core.service.utils.HelperAuthService;
import jakarta.transaction.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.oauth2.client.oidc.userinfo.OidcUserRequest;
import org.springframework.security.oauth2.client.oidc.userinfo.OidcUserService;
import org.springframework.security.oauth2.core.oidc.user.DefaultOidcUser;
import org.springframework.security.oauth2.core.oidc.user.OidcUser;
import org.springframework.stereotype.Service;

import java.util.Set;

@Service
public class HockeyOidcUserService extends OidcUserService {

    private final HelperAuthService<OidcUser> helperAuthService;

    @Autowired
    public HockeyOidcUserService(UserService userService, HockeyUserProperties hockeyUserProperties) {
        this.helperAuthService = new HelperAuthService<>(userService, hockeyUserProperties);
    }

    @Override
    @Transactional
    public OidcUser loadUser(OidcUserRequest userRequest) {
        OidcUser oidcUser = super.loadUser(userRequest);
        System.out.println("\nThis is the TOKEN: " + oidcUser.getIdToken().getTokenValue() + "\n");  // TODO: make this a logger

        String firstName = oidcUser.getAttribute("given_name");
        String lastName = oidcUser.getAttribute("family_name");
        String email = oidcUser.getAttribute("email");

        if (firstName == null || lastName == null || email == null) {
            throw new LoginException("Could not get user info from Google!" +
                    " Make sure you have public name and email set.");
        }

        User user = helperAuthService.getUser(firstName, lastName, email);
        Set<GrantedAuthority> authorities = helperAuthService.getAuthorities(user, oidcUser);
        return new DefaultOidcUser(
                authorities,
                oidcUser.getIdToken(),
                oidcUser.getUserInfo()
        );
    }
}
