package cz.muni.fi.icehockeymanager.core.service;

import cz.muni.fi.icehockeymanager.core.data.domain.*;
import cz.muni.fi.icehockeymanager.core.data.repository.LeagueRepository;
import cz.muni.fi.icehockeymanager.core.exceptions.ResourceNotFoundException;
import jakarta.transaction.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@Transactional
public class LeagueService {

    private final LeagueRepository leagueRepository;

    @Autowired
    public LeagueService(LeagueRepository leagueRepository) {
        this.leagueRepository = leagueRepository;
    }

    public List<League> getAllLeagues() {
        return leagueRepository.findAll();
    }

    public League getLeagueById(Long leagueId) {
        return leagueRepository.findById(leagueId).orElseThrow(() -> new ResourceNotFoundException(
                "League with id " + leagueId + " not found."));
    }

    public League createLeague(League league) {
        return leagueRepository.save(league);
    }

    public void deleteLeague(Long leagueId) {
        if (leagueRepository.deleteLeagueById(leagueId) != 1) {
            throw new ResourceNotFoundException("League with id " + leagueId + " does not exist");
        }
    }

    public List<Team> getTeamsByLeagueId(Long leagueId) {
        League league = getLeagueById(leagueId);
        return league.getEnrollments().stream().map(Enrollment::getTeam).toList();
    }

    public List<Schedule> getSchedulesByLeagueId(Long leagueId) {
        League league = getLeagueById(leagueId);
        return league.getSchedules();
    }
}
