package cz.muni.fi.icehockeymanager.core.service;

import cz.muni.fi.icehockeymanager.core.data.domain.Player;
import cz.muni.fi.icehockeymanager.core.data.repository.PlayerRepository;
import cz.muni.fi.icehockeymanager.core.exceptions.ResourceNotFoundException;
import jakarta.transaction.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@Transactional
public class PlayerService {

    private final PlayerRepository playerRepository;

    @Autowired
    public PlayerService(PlayerRepository playerRepository) {
        this.playerRepository = playerRepository;
    }

    public List<Player> getAllPlayers() {
        return playerRepository.findAll();
    }

    public Player getPlayerById(Long playerId) {
        return playerRepository.findById(playerId).orElseThrow(() -> new ResourceNotFoundException(
                "Player with id " + playerId + " not found."));
    }

    public Player createPlayer(Player player) {
        return playerRepository.save(player);
    }

    public void deletePlayer(Long id) {
        if (playerRepository.deletePlayerById(id) != 1) {
            throw new ResourceNotFoundException("Player with id " + id + " does not exist");
        }
    }
}
