package cz.muni.fi.icehockeymanager.core.service;

import cz.muni.fi.icehockeymanager.core.data.domain.*;
import cz.muni.fi.icehockeymanager.core.data.repository.ScheduleRepository;
import cz.muni.fi.icehockeymanager.core.exceptions.ResourceNotFoundException;
import jakarta.transaction.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

@Service
@Transactional
public class ScheduleService {

    private final ScheduleRepository scheduleRepository;
    private final GameService gameService;
    private final LeagueService leagueService;

    @Autowired
    public ScheduleService(ScheduleRepository scheduleRepository, GameService gameService, LeagueService leagueService) {
        this.scheduleRepository = scheduleRepository;
        this.gameService = gameService;
        this.leagueService = leagueService;
    }

    public List<Schedule> getAllSchedules() {
        return scheduleRepository.findAll();
    }

    public Schedule getScheduleById(Long scheduleId) {
        return scheduleRepository.findById(scheduleId).orElseThrow(() -> new ResourceNotFoundException(
                "Schedule with id " + scheduleId + " not found."));
    }

    private void createGames(Schedule schedule) {
        List<Team> teams = leagueService.getTeamsByLeagueId(schedule.getLeague().getId());

        for (int i = 0; i < teams.size(); i++) {
            for (int j = i + 1; j < teams.size(); j++) {
                if (i != j) {
                    Game game = new Game();
                    game.setHomeTeam(teams.get(i));
                    game.setAwayTeam(teams.get(j));
                    game.setDate(new Date());
                    gameService.createGame(game);

                    schedule.addGame(game);
                }
            }
        }
    }

    public Schedule createSchedule(Schedule schedule) {
        this.createGames(schedule);
        return scheduleRepository.save(schedule);
    }

    public void deleteSchedule(Long id) {
        scheduleRepository.deleteScheduleById(id);
    }
}
