package cz.muni.fi.icehockeymanager.core.service;

import cz.muni.fi.icehockeymanager.core.api.create.CreateTeamDto;
import cz.muni.fi.icehockeymanager.core.data.domain.Player;
import cz.muni.fi.icehockeymanager.core.data.domain.Team;
import cz.muni.fi.icehockeymanager.core.data.domain.User;
import cz.muni.fi.icehockeymanager.core.data.repository.TeamRepository;
import cz.muni.fi.icehockeymanager.core.data.repository.UserRepository;
import cz.muni.fi.icehockeymanager.core.exceptions.ResourceNotFoundException;
import jakarta.transaction.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;

@Service
@Transactional
public class TeamService {

    private final TeamRepository teamRepository;
    private final UserRepository userRepository;

    @Autowired
    public TeamService(TeamRepository teamRepository, UserRepository userRepository) {
        this.teamRepository = teamRepository;
        this.userRepository = userRepository;
    }

    public Team createTeam(Team team) {
        return teamRepository.save(team);
    }

    public List<Team> getAllTeams(Optional<Long> createdById) {
        if (createdById.isPresent()) {
            User creator = userRepository.findById(createdById.get()).orElseThrow(
                    () -> new ResourceNotFoundException("User with id " + createdById + " not found."));
            return creator.getTeams();
        }
        return teamRepository.findAll();
    }

    public Team getTeamById(Long teamId) {
        return teamRepository.findById(teamId).orElseThrow(() -> new ResourceNotFoundException(
                "Team with id " + teamId + " not found."));
    }

    public void deleteTeam(Long teamId) {
        if (teamRepository.deleteTeamById(teamId) != 1) {
            throw new ResourceNotFoundException("Team with id " + teamId + " does not exist");
        }
    }

    public Team addPlayer(Long teamId, Player player) {
        Team team = getTeamById(teamId);
        team.addPlayer(player);
        return teamRepository.save(team);
    }
}
