package cz.muni.fi.icehockeymanager.core.service.utils;

import cz.muni.fi.icehockeymanager.core.config.HockeyUserProperties;
import cz.muni.fi.icehockeymanager.core.data.domain.User;
import cz.muni.fi.icehockeymanager.core.service.UserService;
import jakarta.transaction.Transactional;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.oauth2.core.user.OAuth2User;

import java.util.HashSet;
import java.util.Set;

public class HelperAuthService<T extends OAuth2User> {

    private final UserService userService;
    private final HockeyUserProperties hockeyUserProperties;

    public HelperAuthService(UserService userService, HockeyUserProperties hockeyUserProperties) {
        this.userService = userService;
        this.hockeyUserProperties = hockeyUserProperties;
    }

    @Transactional
    public User getUser(String firstName, String lastName, String email) {
        User user;
        // If user exists in database, do nothing
        if (!userService.existsUserByEmail(email)) {
            if (hockeyUserProperties.getAdminEmails().contains(email)) {
                user = new User(firstName, lastName, email, true);
            } else {
                user = new User(firstName, lastName, email);
            }
            userService.createUser(user);
        } else {
            user = userService.getUserByEmail(email);
        }

        return user;
    }

    @Transactional
    public Set<GrantedAuthority> getAuthorities(User user, T oidcUser) {
        Set<GrantedAuthority> authorities = new HashSet<>(oidcUser.getAuthorities());
        if (user.getIsAdmin()) {
            authorities.add(new SimpleGrantedAuthority("ADMIN"));
        } else {
            authorities.add(new SimpleGrantedAuthority("USER"));
        }

        return authorities;
    }
}
