package cz.muni.fi.icehockeymanager.core.rest;

import cz.muni.fi.icehockeymanager.core.api.create.CreateEnrollmentDto;
import cz.muni.fi.icehockeymanager.core.api.view.EnrollmentDto;
import cz.muni.fi.icehockeymanager.core.facade.EnrollmentFacade;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.security.oauth2.client.registration.ClientRegistrationRepository;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;

import java.util.ArrayList;
import java.util.List;

import static org.mockito.ArgumentMatchers.any;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@WebMvcTest(controllers = EnrollmentRestController.class)
@WithMockUser(value = "spring")
public class EnrollmentRestControllerTest {

    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private EnrollmentFacade enrollmentFacade;

    @MockBean
    private ClientRegistrationRepository clientRegistrationRepository;

    @Test
    public void testListEnrollments() throws Exception {
        EnrollmentDto enrollment = new EnrollmentDto();
        enrollment.setId(1L);
        enrollment.setLeagueId(1L);
        enrollment.setTeamId(1L);

        List<EnrollmentDto> enrollments = new ArrayList<>();
        enrollments.add(enrollment);

        Mockito.when(enrollmentFacade.filterEnrollments(any(), any())).thenReturn(enrollments);

        MvcResult response = mockMvc.perform(get("/enrollments"))
                .andExpect(status().isOk())
                .andReturn();

        Assertions.assertEquals(
                "[{\"id\":1,\"teamId\":1,\"leagueId\":1}]",
                response.getResponse().getContentAsString()
        );
    }

    @Test
    public void testCreateEnrollment() throws Exception {
        EnrollmentDto enrollmentDto = new EnrollmentDto();
        enrollmentDto.setId(1L);
        enrollmentDto.setLeagueId(1L);
        enrollmentDto.setTeamId(1L);

        Mockito.when(enrollmentFacade.createEnrollment(any(CreateEnrollmentDto.class))).thenReturn(enrollmentDto);

        MvcResult response = mockMvc.perform(post("/enrollments")
                .contentType("application/json")
                .content("{\"teamId\":1,\"leagueId\":1}"))
                .andExpect(status().isCreated())
                .andReturn();

        Assertions.assertEquals(
                "{\"id\":1,\"teamId\":1,\"leagueId\":1}",
                response.getResponse().getContentAsString()
        );
    }

    @Test
    public void testDeleteEnrollment() throws Exception {
        mockMvc.perform(delete("/enrollments/1"))
                .andExpect(status().isNoContent());
        Mockito.verify(enrollmentFacade).deleteEnrollment(1L);
    }
}
