package cz.muni.fi.icehockeymanager.core.rest;

import cz.muni.fi.icehockeymanager.core.api.create.CreateGameDto;
import cz.muni.fi.icehockeymanager.core.facade.GameFacade;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.security.oauth2.client.registration.ClientRegistrationRepository;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;

import java.util.Date;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@WebMvcTest(controllers = GameRestController.class)
@WithMockUser(value = "spring", authorities = "ADMIN")
public class GameRestControllerTest {

    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private GameFacade gameFacade;

    @MockBean
    private ClientRegistrationRepository clientRegistrationRepository;

    @Test
    public void testListGames() throws Exception {
        mockMvc.perform(get("/games"))
                .andExpect(status().isOk());

        Mockito.verify(gameFacade).getAllGames();
    }

    @Test
    public void testCreateGame() throws Exception {
        mockMvc.perform(post("/games")
                .content("{\"scheduleId\":1,\"homeTeamId\":1,\"date\":\"2023-04-16T10:00:00+02\",\"awayTeamId\":2}")
                .contentType("application/json"))
                .andExpect(status().isCreated());

        CreateGameDto expectedCreateGameDto = new CreateGameDto();
        expectedCreateGameDto.setHomeTeamId(1L);
        expectedCreateGameDto.setAwayTeamId(2L);

        expectedCreateGameDto.setDate(new Date(2023-1900, 3, 16, 10, 0, 0));
        expectedCreateGameDto.setScheduleId(1L);
        Mockito.verify(gameFacade).createGame(expectedCreateGameDto);
    }

    @Test
    public void testDeleteGame() throws Exception {
        mockMvc.perform(delete("/games/1"))
                .andExpect(status().isNoContent());

        Mockito.verify(gameFacade).deleteGame(1L);
    }

    @Test
    public void testGetGame() throws Exception {
        mockMvc.perform(get("/games/1"))
                .andExpect(status().isOk());

        Mockito.verify(gameFacade).getGameById(1L);
    }

    @Test
    public void testStartGame() throws Exception {
        mockMvc.perform(patch("/games/1"))
                .andExpect(status().isOk());

        Mockito.verify(gameFacade).startGame(1L);
    }
}
