package cz.muni.fi.icehockeymanager.core.rest;

import cz.muni.fi.icehockeymanager.core.api.view.PlayerDto;
import cz.muni.fi.icehockeymanager.core.data.enums.PlayerPosition;
import cz.muni.fi.icehockeymanager.core.facade.PlayerFacade;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.security.oauth2.client.registration.ClientRegistrationRepository;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;

import java.util.ArrayList;
import java.util.List;

import static org.mockito.ArgumentMatchers.any;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@WebMvcTest(controllers = PlayerRestController.class)
@WithMockUser(value = "spring", authorities = "ADMIN")
public class PlayerRestControllerTest {

    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private PlayerFacade playerFacade;

    @MockBean
    private ClientRegistrationRepository clientRegistrationRepository;

    @Test
    public void testListPlayers() throws Exception {
        PlayerDto player = new PlayerDto();
        player.setId(1L);
        player.setFirstName("John");
        player.setLastName("Doe");
        player.setStrength(10);
        player.setPosition(PlayerPosition.FORWARD);

        List<PlayerDto> players = new ArrayList<>();
        players.add(player);

        Mockito.when(playerFacade.getAllPlayers()).thenReturn(players);

        MvcResult response = mockMvc.perform(get("/players"))
                .andExpect(status().isOk())
                .andReturn();

        Assertions.assertEquals(
                "[{\"id\":1,\"firstName\":\"John\",\"lastName\":\"Doe\",\"strength\":10,\"position\":\"FORWARD\",\"teamId\":null}]",
                response.getResponse().getContentAsString()
        );
    }

    @Test
    public void testCreatePlayer() throws Exception {
        PlayerDto playerDto = new PlayerDto();
        playerDto.setId(1L);
        playerDto.setFirstName("John");
        playerDto.setLastName("Doe");
        playerDto.setStrength(10);
        playerDto.setPosition(PlayerPosition.FORWARD);

        Mockito.when(playerFacade.createPlayer(any())).thenReturn(playerDto);

        MvcResult response = mockMvc.perform(
                post("/players")
                        .content("{\"firstName\":\"John\",\"lastName\":\"Doe\",\"strength\":10,\"position\":\"FORWARD\"}")
                        .contentType("application/json")
                )
                .andExpect(status().isCreated())
                .andReturn();

        Assertions.assertEquals(
                "{\"id\":1,\"firstName\":\"John\",\"lastName\":\"Doe\",\"strength\":10,\"position\":\"FORWARD\",\"teamId\":null}",
                response.getResponse().getContentAsString()
        );
    }

    @Test
    public void testDeletePlayer() throws Exception {
        mockMvc.perform(delete("/players/1"))
                .andExpect(status().isNoContent());
        Mockito.verify(playerFacade).deletePlayer(1L);
    }
}
