package cz.muni.fi.icehockeymanager.core.rest;

import cz.muni.fi.icehockeymanager.core.api.create.CreateScheduleDto;
import cz.muni.fi.icehockeymanager.core.api.view.LeagueDto;
import cz.muni.fi.icehockeymanager.core.api.view.ScheduleDetailDto;
import cz.muni.fi.icehockeymanager.core.api.view.ScheduleDto;
import cz.muni.fi.icehockeymanager.core.facade.ScheduleFacade;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.security.oauth2.client.registration.ClientRegistrationRepository;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;

import java.util.ArrayList;
import java.util.List;

import static org.mockito.ArgumentMatchers.any;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@WebMvcTest(controllers = ScheduleRestController.class)
@WithMockUser(value = "spring", authorities = "ADMIN")
public class ScheduleRestControllerTest {

    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private ScheduleFacade scheduleFacade;

    @MockBean
    private ClientRegistrationRepository clientRegistrationRepository;

    @Test
    public void testListSchedules() throws Exception {
        ScheduleDto scheduleDto = new ScheduleDto();
        scheduleDto.setId(1L);
        scheduleDto.setYear(2014);
        scheduleDto.setLeagueId(1L);

        List<ScheduleDto> schedule = new ArrayList<>();
        schedule.add(scheduleDto);

        Mockito.when(scheduleFacade.getAllSchedules()).thenReturn(schedule);

        MvcResult response = mockMvc.perform(get("/schedules"))
                .andExpect(status().isOk())
                .andReturn();

        Assertions.assertEquals(
                "[{\"id\":1,\"year\":2014,\"leagueId\":1}]",
                response.getResponse().getContentAsString()
        );
    }

    @Test
    public void testCreateSchedule() throws Exception {
        LeagueDto leagueDto = new LeagueDto();
        leagueDto.setId(1L);
        leagueDto.setName("League");
        leagueDto.setAcronym("L");

        ScheduleDetailDto scheduleDetailDto = new ScheduleDetailDto();
        scheduleDetailDto.setId(1L);
        scheduleDetailDto.setLeague(leagueDto);

        Mockito.when(scheduleFacade.createSchedule(any(CreateScheduleDto.class))).thenReturn(scheduleDetailDto);

        MvcResult response = mockMvc.perform(post("/schedules")
                .content("{\"leagueId\":1}")
                .contentType("application/json"))
                .andExpect(status().isCreated())
                .andReturn();

        Assertions.assertEquals(
                "{\"id\":1,\"year\":0,\"league\":{\"id\":1,\"name\":\"League\",\"acronym\":\"L\"},\"games\":[]}",
                response.getResponse().getContentAsString()
        );
    }

    @Test
    public void testDeleteSchedule() throws Exception {
        mockMvc.perform(delete("/schedules/1"))
                .andExpect(status().isNoContent());

        Mockito.verify(scheduleFacade).deleteSchedule(1L);
    }
}
