package cz.muni.fi.icehockeymanager.core.rest;

import cz.muni.fi.icehockeymanager.core.api.create.CreateUserDto;
import cz.muni.fi.icehockeymanager.core.api.view.UserDto;
import cz.muni.fi.icehockeymanager.core.facade.UserFacade;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.security.oauth2.client.registration.ClientRegistrationRepository;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;

import java.util.ArrayList;
import java.util.List;

import static org.mockito.ArgumentMatchers.any;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@WebMvcTest(controllers = UserRestController.class)
@WithMockUser(value = "spring", authorities = "ADMIN")
public class UserRestControllerTest {

    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private UserFacade userFacade;

    @MockBean
    private ClientRegistrationRepository clientRegistrationRepository;

    @Test
    public void testListUsers() throws Exception {
        UserDto user = new UserDto();
        user.setId(1L);
        user.setFirstName("John");
        user.setLastName("Doe");
        user.setEmail("name@email.com");
        user.setIsAdmin(true);

        List<UserDto> users = new ArrayList<>();
        users.add(user);

        Mockito.when(userFacade.getAllUsers()).thenReturn(users);

        MvcResult response = mockMvc.perform(get("/users"))
                .andExpect(status().isOk()).andReturn();

        Assertions.assertEquals(
                "[{\"id\":1,\"firstName\":\"John\",\"lastName\":\"Doe\",\"email\":\"name@email.com\",\"isAdmin\":true}]",
                response.getResponse().getContentAsString()
        );
    }

    @Test
    public void testRegisterUser() throws Exception {
        UserDto user = new UserDto();
        user.setId(1L);
        user.setFirstName("John");
        user.setLastName("Doe");
        user.setEmail("name@email.com");
        user.setIsAdmin(false);

        Mockito.when(userFacade.registerUser(any(CreateUserDto.class))).thenReturn(user);

        MvcResult response = mockMvc.perform(post("/users")
                        .contentType("application/json")
                        .content("{\"firstName\":\"John\",\"lastName\":\"Doe\",\"email\":\"name@email.com\"}"))
                .andExpect(status().isCreated())
                .andReturn();

        Assertions.assertEquals(
                "{\"id\":1,\"firstName\":\"John\",\"lastName\":\"Doe\",\"email\":\"name@email.com\",\"isAdmin\":false}",
                response.getResponse().getContentAsString()
        );
    }

    @Test
    public void testDeleteUser() throws Exception {
        mockMvc.perform(delete("/users/1"))
                .andExpect(status().isNoContent());
        Mockito.verify(userFacade).deleteUser(1L);
    }

    @Test
    public void testGetUser() throws Exception {
        UserDto user = new UserDto();
        user.setId(1L);
        user.setFirstName("John");
        user.setLastName("Doe");
        user.setEmail("name@email.com");
        user.setIsAdmin(false);

        Mockito.when(userFacade.getUserById(1L)).thenReturn(user);

        MvcResult response = mockMvc.perform(get("/users/1"))
                .andExpect(status().isOk())
                .andReturn();

        Assertions.assertEquals(
                "{\"id\":1,\"firstName\":\"John\",\"lastName\":\"Doe\",\"email\":\"name@email.com\",\"isAdmin\":false}",
                response.getResponse().getContentAsString()
        );
    }
}
