package cz.muni.fi.icehockeymanager.core.service;

import cz.muni.fi.icehockeymanager.core.data.domain.Enrollment;
import cz.muni.fi.icehockeymanager.core.data.domain.League;
import cz.muni.fi.icehockeymanager.core.data.domain.Team;
import jakarta.transaction.Transactional;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.security.oauth2.client.registration.ClientRegistrationRepository;

import java.util.List;
import java.util.Optional;


@SpringBootTest
@Transactional
public class EnrollmentServiceTest {

    @Autowired
    private EnrollmentService enrollmentService;

    @Autowired
    private TeamService teamService;

    @Autowired
    private LeagueService leagueService;

    @MockBean
    private ClientRegistrationRepository clientRegistrationRepository;

    @Test
    public void testGetAllEnrollments() {
        List<Enrollment> enrollments = enrollmentService.filterEnrollments(Optional.empty(), Optional.empty());

        assert enrollments.size() == 7;
        assert enrollments.get(0).getTeam().equals(teamService.getTeamById(1L));
        assert enrollments.get(0).getLeague().equals(leagueService.getLeagueById(1L));
        assert enrollments.get(1).getTeam().equals(teamService.getTeamById(1L));
        assert enrollments.get(1).getLeague().equals(leagueService.getLeagueById(2L));
        assert enrollments.get(2).getTeam().equals(teamService.getTeamById(2L));
        assert enrollments.get(2).getLeague().equals(leagueService.getLeagueById(4L));
        assert enrollments.get(3).getTeam().equals(teamService.getTeamById(2L));
        assert enrollments.get(3).getLeague().equals(leagueService.getLeagueById(1L));
        assert enrollments.get(4).getTeam().equals(teamService.getTeamById(3L));
        assert enrollments.get(4).getLeague().equals(leagueService.getLeagueById(1L));
    }

    @Test
    public void testFilterEnrollment() {
        // by teamId //
        List<Enrollment> enrollments = enrollmentService.filterEnrollments(Optional.of(1L), Optional.empty());

        assert enrollments.size() == 3;
        assert enrollments.stream().filter(e -> e.getId() == 1L).toList().size() == 1;
        assert enrollments.stream().filter(e -> e.getId() == 2L).toList().size() == 1;

        Enrollment enrollment1 = enrollments.stream().filter(e -> e.getId() == 1L).toList().get(0);
        Enrollment enrollment2 = enrollments.stream().filter(e -> e.getId() == 2L).toList().get(0);

        assert enrollment1.getTeam().equals(teamService.getTeamById(1L));
        assert enrollment1.getLeague().equals(leagueService.getLeagueById(1L));
        assert enrollment2.getTeam().equals(teamService.getTeamById(1L));
        assert enrollment2.getLeague().equals(leagueService.getLeagueById(2L));

        // by leagueId //
        enrollments = enrollmentService.filterEnrollments(Optional.empty(), Optional.of(4L));

        assert enrollments.size() == 1;
        assert enrollments.get(0).getId().equals(3L);
        assert enrollments.get(0).getTeam().equals(teamService.getTeamById(2L));
        assert enrollments.get(0).getLeague().equals(leagueService.getLeagueById(4L));
    }

    @Test
    public void testCreateEnrollment() {
        League league = leagueService.getLeagueById(1L);
        Team team = teamService.getTeamById(1L);

        Enrollment enrollment = new Enrollment();
        enrollment.setLeague(league);
        enrollment.setTeam(team);

        enrollmentService.createEnrollment(enrollment);

        List<Enrollment> enrollments = enrollmentService.filterEnrollments(Optional.empty(), Optional.empty());
        assert enrollments.size() == 8;
        assert enrollments.stream().filter(e -> e.getId() == 8).toList().get(0).getLeague().equals(league);
        assert enrollments.stream().filter(e -> e.getId() == 8).toList().get(0).getTeam().equals(team);
    }

    @Test
    public void testDeleteEnrollment() {
        enrollmentService.deleteEnrollment(1L);
        List<Enrollment> enrollments = enrollmentService.filterEnrollments(Optional.empty(), Optional.empty());

        assert enrollments.size() == 6;
    }
}
