package cz.muni.fi.icehockeymanager.core.service;

import cz.muni.fi.icehockeymanager.core.data.domain.Game;
import cz.muni.fi.icehockeymanager.core.data.enums.GameStatus;
import jakarta.transaction.Transactional;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.security.oauth2.client.registration.ClientRegistrationRepository;

import java.util.Calendar;
import java.util.Date;
import java.util.List;

@SpringBootTest
@Transactional
public class GameServiceTest {

    @Autowired
    private GameService gameService;

    @Autowired
    private TeamService teamService;

    @Autowired
    private ScheduleService scheduleService;

    @MockBean
    private ClientRegistrationRepository clientRegistrationRepository;
    
    private final Date date1;
    private final Date date2;

    public GameServiceTest() {
        Calendar calendar = Calendar.getInstance();

        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);

        calendar.set(Calendar.YEAR, 2023);
        calendar.set(Calendar.MONTH, 2);
        calendar.set(Calendar.DAY_OF_MONTH, 25);

        date1 = calendar.getTime();

        calendar.set(Calendar.YEAR, 2023);
        calendar.set(Calendar.MONTH, 4);
        calendar.set(Calendar.DAY_OF_MONTH, 16);

        date2 = calendar.getTime();
    }

    @Test
    public void testGetAllGames() {
        List<Game> games = gameService.getAllGames();

        assert games.size() == 1;
        assert games.get(0).getStatus().equals(GameStatus.FINISHED);
        assert games.get(0).getDate().compareTo(date1) == 0;
        assert games.get(0).getHomeTeam().equals(teamService.getTeamById(1L));
        assert games.get(0).getAwayTeam().equals(teamService.getTeamById(2L));
        assert games.get(0).getHomeTeamGoals().equals(3);
        assert games.get(0).getAwayTeamGoals().equals(4);
        assert games.get(0).getSchedule().equals(scheduleService.getScheduleById(1L));
    }

    @Test
    public void testCreateGame() {
        Game game = new Game();
        game.setStatus(GameStatus.IN_PROGRESS);
        game.setDate(date2);
        game.setHomeTeam(teamService.getTeamById(1L));
        game.setAwayTeam(teamService.getTeamById(2L));
        game.setHomeTeamGoals(3);
        game.setAwayTeamGoals(1);
        game.setSchedule(scheduleService.getScheduleById(1L));

        gameService.createGame(game);

        List<Game> games = gameService.getAllGames();
        assert games.get(1).getStatus().equals(GameStatus.IN_PROGRESS);
        assert games.get(1).getDate().compareTo(date2) == 0;
        assert games.get(1).getHomeTeam().equals(teamService.getTeamById(1L));
        assert games.get(1).getAwayTeam().equals(teamService.getTeamById(2L));
        assert games.get(1).getHomeTeamGoals().equals(3);
        assert games.get(1).getAwayTeamGoals().equals(1);
        assert games.get(1).getSchedule().equals(scheduleService.getScheduleById(1L));
    }

    @Test
    public void testDeleteGame() {
        gameService.deleteGame(1L);
        List<Game> games = gameService.getAllGames();

        assert games.size() == 0;
    }

    @Test
    public void testGetGameById() {
        Game game = gameService.getGameById(1L);

        assert game.getStatus().equals(GameStatus.FINISHED);
        assert game.getDate().compareTo(date1) == 0;
        assert game.getHomeTeam().equals(teamService.getTeamById(1L));
        assert game.getAwayTeam().equals(teamService.getTeamById(2L));
        assert game.getHomeTeamGoals().equals(3);
        assert game.getAwayTeamGoals().equals(4);
        assert game.getSchedule().equals(scheduleService.getScheduleById(1L));
    }
}
