package cz.muni.fi.icehockeymanager.core.service;

import cz.muni.fi.icehockeymanager.core.data.domain.League;
import cz.muni.fi.icehockeymanager.core.data.domain.Schedule;
import jakarta.transaction.Transactional;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.security.oauth2.client.registration.ClientRegistrationRepository;

import java.util.ArrayList;
import java.util.List;

@SpringBootTest
@Transactional
public class LeagueServiceTest {

    @Autowired
    private LeagueService leagueService;

    @Autowired
    private ScheduleService scheduleService;

    @MockBean
    private ClientRegistrationRepository clientRegistrationRepository;

    @Test
    public void testGetAllLeagues() {
        List<League> leagues = leagueService.getAllLeagues();

        assert leagues.size() == 5;
        assert leagues.get(0).getName().equals("National Hockey League");
        assert leagues.get(0).getAcronym().equals("NHL");
        assert leagues.get(0).getSchedules().get(0).equals(scheduleService.getScheduleById(1L));
    }

    @Test
    public void testCreateLeague() {
        Schedule schedule = scheduleService.getScheduleById(2L);

        League league = new League();
        league.setName("Kontinental Hockey League");
        league.setAcronym("KHL");
        league.setSchedules(new ArrayList<>());
        league.addSchedule(schedule);

        leagueService.createLeague(league);

        List<League> leagues = leagueService.getAllLeagues();
        assert leagues.size() == 6;
        assert leagues.stream().filter(l -> l.getId() == 6).toList().get(0).getName().equals("Kontinental Hockey League");
        assert leagues.stream().filter(l -> l.getId() == 6).toList().get(0).getAcronym().equals("KHL");
        assert leagues.stream().filter(l -> l.getId() == 6).toList().get(0).getSchedules().get(0).equals(schedule);
    }

    @Test
    public void testDeleteLeague() {
        leagueService.deleteLeague(3L);
        List<League> leagues = leagueService.getAllLeagues();

        assert leagues.size() == 4;
    }

    @Test
    public void testGetLeagueById() {
        League league = leagueService.getLeagueById(1L);

        assert league.getName().equals("National Hockey League");
        assert league.getAcronym().equals("NHL");
        assert league.getSchedules().get(0).equals(scheduleService.getScheduleById(1L));
    }
}
