package cz.muni.fi.icehockeymanager.core.service;

import cz.muni.fi.icehockeymanager.core.data.domain.Player;
import cz.muni.fi.icehockeymanager.core.data.domain.Team;
import cz.muni.fi.icehockeymanager.core.data.enums.PlayerPosition;
import jakarta.transaction.Transactional;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.security.oauth2.client.registration.ClientRegistrationRepository;

import java.util.List;

@SpringBootTest
@Transactional
public class PlayerServiceTest {

    @Autowired
    private PlayerService playerService;

    @Autowired
    private TeamService teamService;

    @MockBean
    private ClientRegistrationRepository clientRegistrationRepository;

    @Test
    public void testGetAllPlayers() {
        List<Player> players = playerService.getAllPlayers();

        assert players.size() == 1;
        assert players.get(0).getFirstName().equals("Alexander");
        assert players.get(0).getLastName().equals("Ovechkin");
        assert players.get(0).getStrength().equals(95);
        assert players.get(0).getPosition().equals(PlayerPosition.FORWARD);
        assert players.get(0).getTeam().equals(teamService.getTeamById(1L));
    }

    @Test
    public void testCreatePlayer() {
        Team team = teamService.getTeamById(1L);

        Player player = new Player();
        player.setFirstName("Ryan");
        player.setLastName("O'Reilly");
        player.setStrength(90);
        player.setPosition(PlayerPosition.DEFENDER);
        player.setTeam(team);

        playerService.createPlayer(player);

        List<Player> players = playerService.getAllPlayers();
        assert players.size() == 2;
        assert players.get(1).getFirstName().equals("Ryan");
        assert players.get(1).getLastName().equals("O'Reilly");
        assert players.get(1).getStrength().equals(90);
        assert players.get(1).getPosition().equals(PlayerPosition.DEFENDER);
        assert players.get(1).getTeam().equals(team);
    }

    @Test
    public void testDeletePlayer() {
        playerService.deletePlayer(1L);
        List<Player> players = playerService.getAllPlayers();

        assert players.size() == 0;
    }
}
