package cz.muni.fi.icehockeymanager.core.service;

import cz.muni.fi.icehockeymanager.core.data.domain.League;
import cz.muni.fi.icehockeymanager.core.data.domain.Schedule;
import jakarta.transaction.Transactional;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.security.oauth2.client.registration.ClientRegistrationRepository;

import java.util.Comparator;
import java.util.List;


@SpringBootTest
@Transactional
public class ScheduleServiceTest {

    @Autowired
    private ScheduleService scheduleService;

    @Autowired
    private LeagueService leagueService;

    @Autowired
    private GameService gameService;

    @MockBean
    private ClientRegistrationRepository clientRegistrationRepository;

    @Test
    public void testGetAllSchedules() {
        List<Schedule> schedules = sortSchedules(scheduleService.getAllSchedules());

        assert schedules.size() == 3;
        assert schedules.get(0).getLeague().equals(leagueService.getLeagueById(1L));
    }

    @Test
    public void testCreateSchedule() {
        // test schedule //
        League league = leagueService.getLeagueById(5L);

        Schedule schedule = new Schedule();
        schedule.setLeague(league);

        scheduleService.createSchedule(schedule);

        List<Schedule> schedules = sortSchedules(scheduleService.getAllSchedules());

        assert schedules.size() == 4;
        assert schedules.get(3).getLeague().equals(league);

        assert schedule.getGames().size() == 1;
    }

    @Test
    public void testDeleteSchedule() {
        scheduleService.deleteSchedule(2L);
        List<Schedule> schedules = sortSchedules(scheduleService.getAllSchedules());

        assert schedules.size() == 2;
    }

    @Test
    public void testGetScheduleById() {
        Schedule schedule = scheduleService.getScheduleById(1L);

        assert schedule.getId().equals(1L);
        assert schedule.getLeague().getId().equals(leagueService.getLeagueById(1L).getId());
    }

    private List<Schedule> sortSchedules(List<Schedule> schedules) {
        return schedules.stream().sorted(Comparator.comparingLong(Schedule::getId)).toList();
    }
}
