package cz.muni.fi.icehockeymanager.core.service;

import cz.muni.fi.icehockeymanager.core.data.domain.Team;
import cz.muni.fi.icehockeymanager.core.data.domain.User;
import jakarta.transaction.Transactional;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.security.oauth2.client.registration.ClientRegistrationRepository;

import java.util.List;
import java.util.Optional;

@SpringBootTest
@Transactional
public class TeamServiceTest {

    @Autowired
    private TeamService teamService;

    @Autowired
    private UserService userService;

    @MockBean
    private ClientRegistrationRepository clientRegistrationRepository;

    @Test
    public void testGetAllTeams() {
        // without createdById //
        List<Team> teams = teamService.getAllTeams(Optional.empty());

        assert teams.size() == 4;
        assert teams.get(0).getName().equals("Washington Capitals");
        assert teams.get(0).getCreatedBy().equals(userService.getUserById(1L));
        assert teams.get(1).getName().equals("Boston Bruins");
        assert teams.get(1).getCreatedBy().equals(userService.getUserById(2L));

        // with createdById
        teams = teamService.getAllTeams(Optional.of(2L));

        assert teams.size() == 3;
        assert teams.get(0).getName().equals("Boston Bruins");
        assert teams.get(0).getCreatedBy().equals(userService.getUserById(2L));
        assert teams.get(1).getName().equals("Montreal Canadiens");
        assert teams.get(1).getCreatedBy().equals(userService.getUserById(2L));
        assert teams.get(2).getName().equals("Minnesota Wild");
        assert teams.get(2).getCreatedBy().equals(userService.getUserById(2L));
    }

    @Test
    public void testCreateTeam() {
        User createdByUser = userService.getUserById(1L);

        Team team = new Team();
        team.setName("Winnipeg Jets");
        team.setCreatedBy(createdByUser);

        teamService.createTeam(team);

        List<Team> teams = teamService.getAllTeams(Optional.empty());
        assert teams.size() == 5;
        assert teams.get(4).getName().equals("Winnipeg Jets");
        assert teams.get(4).getCreatedBy().equals(createdByUser);
    }

    @Test
    public void testDeleteTeam() {
        teamService.deleteTeam(4L);
        List<Team> teams = teamService.getAllTeams(Optional.empty());

        assert teams.size() == 3;
    }

    @Test
    public void testGetTeamById() {
        Team team = teamService.getTeamById(1L);

        assert team.getName().equals("Washington Capitals");
        assert team.getCreatedBy().getId().equals(1L);
    }
}
