package cz.muni.fi.icehockeymanager.core.service;

import cz.muni.fi.icehockeymanager.core.data.domain.User;
import jakarta.transaction.Transactional;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.security.oauth2.client.registration.ClientRegistrationRepository;

import java.util.List;

@SpringBootTest
@Transactional
public class UserServiceTest {

    @Autowired
    private UserService userService;

    @MockBean
    private ClientRegistrationRepository clientRegistrationRepository;

    @Test
    public void testGetAllUsers() {
        List<User> users = userService.getAllUsers();
        assert users.size() == 3;
        assert users.get(0).getFirstName().equals("John");
        assert users.get(0).getLastName().equals("Doe");
        assert users.get(1).getFirstName().equals("Kara");
        assert users.get(1).getLastName().equals("Thrace");
        assert users.get(2).getFirstName().equals("Lee");
        assert users.get(2).getLastName().equals("Adama");
    }

    @Test
    public void testCreateUser() {
        User user = new User();
        user.setFirstName("Jane");
        user.setLastName("Doe");
        user.setEmail("jane@email.com");
        user.setIsAdmin(false);

        userService.createUser(user);
        List<User> users = userService.getAllUsers();
        assert users.size() == 4;
        assert users.get(3).getFirstName().equals("Jane");
        assert users.get(3).getLastName().equals("Doe");
    }

    @Test
    public void testDeleteUser() {
        userService.deleteUser(3L);
        List<User> users = userService.getAllUsers();
        assert users.size() == 2;
    }

    @Test
    public void testGetUserById() {
        User user = userService.getUserById(1L);
        assert user.getFirstName().equals("John");
        assert user.getLastName().equals("Doe");
    }
}
