CREATE TABLE hockey_user (
    id SERIAL PRIMARY KEY,
    first_name VARCHAR(50),
    last_name VARCHAR(50),
    email VARCHAR(50) NOT NULL UNIQUE,
    is_admin BOOLEAN NOT NULL DEFAULT FALSE
);

CREATE TABLE team (
    id SERIAL PRIMARY KEY,
    name VARCHAR(50),
    created_by BIGINT REFERENCES hockey_user(id) REFERENCES hockey_user(id) ON DELETE SET NULL
);

CREATE TABLE league (
    id SERIAL PRIMARY KEY,
    name VARCHAR(100),
    acronym VARCHAR(10)
);

CREATE TABLE schedule (
    id SERIAL PRIMARY KEY,
    schedule_year INT NOT NULL,
    league_id BIGINT REFERENCES league (id)
);



CREATE TABLE enrollment (
    id SERIAL PRIMARY KEY,
    team_id BIGINT REFERENCES team (id),
    league_id BIGINT REFERENCES league (id)
);

CREATE TABLE game (
    id SERIAL PRIMARY KEY,
    status VARCHAR(255),
    date DATE,
    home_team_goals INTEGER,
    away_team_goals INTEGER,
    home_team_id BIGINT REFERENCES team(id),
    away_team_id BIGINT REFERENCES team(id),
    schedule_id BIGINT REFERENCES schedule(id)
);

CREATE TABLE player (
    id SERIAL PRIMARY KEY,
    first_name VARCHAR(50),
    last_name VARCHAR(50),
    strength INTEGER CONSTRAINT strength_range CHECK (strength >= 1 AND strength <= 100),
    position VARCHAR(255),
    team_id BIGINT REFERENCES team(id)
);