import datetime
import random
import time

from locust import HttpUser, task, between, SequentialTaskSet
import hosts
from faker import Faker


class User(HttpUser):
    @task
    class AdminSequence(SequentialTaskSet):
        wait_time = between(1, 2)

        @task
        def create_schedules_for_leagues(self):
            while True:
                response = self.client.get(hosts.hockey_manager + "/leagues")
                if response.status_code == hosts.HTTP_STATUS_OK:
                    break
                time.sleep(1)
            leagues = response.json()

            for league in leagues:
                games = self.create_schedule_and_games(league)
                self.create_reservations_for_league(games)
            self.create_and_notify_subscriber()
            self.parent.environment.runner.stop()

        def create_schedule_and_games(self, league):
            games = []
            fake = Faker()
            while True:
                response = self.client.post(hosts.hockey_manager + "/schedules",
                                            json={
                                                "leagueId": int(league["id"]),
                                                "year": datetime.datetime.now().year
                                            })
                if response.status_code == hosts.HTTP_STATUS_CREATED:
                    break
                time.sleep(1)
            schedule = response.json()

            while True:
                response = self.client.get(hosts.hockey_manager + "/leagues/" + str(league["id"]))
                if response.status_code == hosts.HTTP_STATUS_OK:
                    break
                time.sleep(1)

            teams_in_league = response.json()["teams"]

            for first_team in teams_in_league:
                for second_team in teams_in_league:
                    if first_team["id"] == second_team["id"]:
                        continue
                    rand_start_time = fake.date_time_between(start_date='now', end_date='+200d')
                    response = self.client.post(hosts.hockey_manager + "/games",
                                                json={
                                                    "homeTeamId": int(first_team["id"]),
                                                    "awayTeamId": int(second_team["id"]),
                                                    "date": datetime.datetime(rand_start_time.year,
                                                                              rand_start_time.month,
                                                                              rand_start_time.day,
                                                                              19, 0, 0, 0).isoformat(),
                                                    "scheduleId": int(schedule["id"])
                                                })
                    if response.status_code == hosts.HTTP_STATUS_CREATED:
                        games.append(response.json())

            return games

        def create_reservations_for_league(self, games):
            while True:
                response = self.client.get(hosts.arena_reservation + "/arenas")
                if response.status_code == hosts.HTTP_STATUS_OK:
                    arenas = response.json()
                    break
                time.sleep(1)

            for game in games:
                arena = random.choice(arenas)
                date = datetime.datetime.fromisoformat(game["date"])
                self.client.post(hosts.arena_reservation + "/reservations/" + str(arena["id"]),
                                 json={
                                     "contactPersonId": 1,
                                     "dateTimeFrom": date.isoformat(),
                                     "dateTimeTo": date.isoformat().replace("19:00:00", "22:00:00"),
                                     "reservationType": "LEAGUE_MATCH"
                                 })

        def create_and_notify_subscriber(self):
            while True:
                response = self.client.post(hosts.report + "/subscribers",
                                            json={
                                                "name": "Mark",
                                                "mail": "mark.c12h22o11berg@gmail.com",
                                                "playerRefId": 10,
                                                "subscribedSubjects": [
                                                    "NEW_GAME_SCHEDULED"
                                                ]
                                            })
                if response.status_code == hosts.HTTP_STATUS_CREATED:
                    subscriber_id = int(response.json()["id"])
                    break
                time.sleep(1)

            while True:
                response = self.client.post(hosts.report + "/notify/" + str(subscriber_id),
                                            json="NEW_GAME_SCHEDULED")
                if response.status_code == hosts.HTTP_STATUS_OK:
                    break
                time.sleep(1)
