import datetime
import random
import time

from locust import HttpUser, task, between, SequentialTaskSet
import dummy_collections
import hosts
from faker import Faker


class User(HttpUser):
    @task
    class ManagerSequence(SequentialTaskSet):
        wait_time = between(1, 1)
        usr_id = 0
        team_id = 0
        name = str()
        surname = str()

        player_count = 20

        @task
        def register_core(self):
            exit_flag = False
            while not exit_flag:
                self.name = random.choice(dummy_collections.names)
                self.surname = random.choice(dummy_collections.surnames)

                response = self.client.post(hosts.hockey_manager + "/users",
                                            json={
                                                "firstName": self.name,
                                                "lastName": self.surname,
                                                "email": str(self.name + self.surname +
                                                             str(random.randint(100, 999)) + "@fastmail.net")
                                            })
                if response.status_code != hosts.HTTP_STATUS_CREATED:
                    continue
                self.usr_id = int(response.json()["id"])
                exit_flag = True

        @task
        def create_team(self):
            exit_flag = False
            while not exit_flag:
                response = self.client.post(hosts.hockey_manager + "/teams",
                                            json={
                                                "name": str("HK " + random.choice(dummy_collections.cities) + " " +
                                                            random.choice(dummy_collections.things)),
                                                "createdById": self.usr_id
                                            })
                if response.status_code != hosts.HTTP_STATUS_CREATED:
                    continue
                self.team_id = int(response.json()["id"])
                exit_flag = True

        def publish_player_info(self, player):
            response = self.client.get(hosts.discussion_forum + "/categories")
            if response.status_code != hosts.HTTP_STATUS_OK:
                return

            response_list = response.json()
            category_id = 0
            for response in response_list:
                if response["name"] == "My Team":
                    category_id = int(response["id"])
                    break

            if category_id == 0:
                response = self.client.post(hosts.discussion_forum + "/categories",
                                            json={
                                                "name": str("My Team")
                                            })
                if response.status_code != hosts.HTTP_STATUS_CREATED:
                    return
                category_id = int(response.json()["id"])

            response = self.client.get(hosts.discussion_forum + "/categories/" + str(category_id))
            if response.status_code != hosts.HTTP_STATUS_OK:
                return

            threads = response.json()["threads"]
            thread_id = 0
            for thread in threads:
                if int(thread["authorId"]) == self.usr_id:
                    thread_id = thread["id"]

            if thread_id == 0:
                response = self.client.post(hosts.discussion_forum + "/threads",
                                            json={
                                                "name": str("Team " + str(self.team_id)),
                                                "description": "This team will surely win anything",
                                                "authorId": self.usr_id,
                                                "categoryId": category_id
                                            })
                if response.status_code != hosts.HTTP_STATUS_CREATED:
                    return
                thread_id = int(response.json()["id"])

            if player["strength"] > 60:
                comment_content = "Checkout the new " + player["position"] + " which I have just added to my team >> " \
                                  + player["firstName"] + " " + player["lastName"] + \
                                  ". His stats are out of this world!!! " + random.choice(
                    dummy_collections.positive_reactions)
            else:
                comment_content = "Hello, I am happy to announce that we have added a new member " \
                                  "to our ice hockey team " + player["firstName"] + " " + player["lastName"] \
                                  + ". He will be playing " + player["position"] + " position. " \
                                  + random.choice(dummy_collections.neutral_reactions)

            self.client.post(hosts.discussion_forum + "/comments",
                             json={
                                 "content": comment_content,
                                 "authorId": self.usr_id,
                                 "threadId": thread_id,
                                 "parentId": 0
                             })

        @task
        def add_players(self):
            while self.player_count > 0:
                response = self.client.get(hosts.hockey_manager + "/players")
                if response.status_code != hosts.HTTP_STATUS_OK:
                    time.sleep(1)
                    continue
                random_player = random.choice(response.json())
                response = self.client.patch(hosts.hockey_manager + "/teams/" + str(self.team_id) +
                                             "/players/" + str(random_player["id"]))
                if response.status_code == hosts.HTTP_STATUS_OK:
                    self.player_count -= 1
                    self.publish_player_info(random_player)

            self.enroll_to_league()
            self.reserve_arenas_for_training()
            self.parent.environment.runner.stop()

        def enroll_to_league(self):
            leagues = self.client.get(hosts.hockey_manager + "/leagues")
            exit_flag = False

            while not exit_flag:
                league = random.choice(leagues.json())
                response = self.client.post(hosts.hockey_manager + "/enrollments",
                                            json={
                                                "teamId": self.team_id,
                                                "leagueId": int(league["id"])
                                            })
                if response.status_code == hosts.HTTP_STATUS_CREATED:
                    exit_flag = True

        def reserve_arenas_for_training(self):
            exit_flag = False
            contact_person_id = 0
            while not exit_flag:
                response = self.client.post(hosts.arena_reservation + "/people",
                                            json={
                                                "name": self.name,
                                                "surname": self.surname,
                                                "email": str(self.name + self.surname +
                                                             str(random.randint(100, 999)) + "@fastmail.net"),
                                                "telephoneNumber": str("+" + str(random.randint(100, 999))
                                                                       + str(random.randint(100, 999))
                                                                       + str(random.randint(100, 999)))
                                            })
                if response.status_code != hosts.HTTP_STATUS_CREATED:
                    continue
                contact_person_id = response.json()["id"]
                exit_flag = True

            self.create_reservation(contact_person_id, 13, "TRAINING")

        def create_reservation(self, contact_person_id, number_of_reservations, reservation_type):
            response = self.client.get(hosts.arena_reservation + "/arenas")
            if response.status_code != hosts.HTTP_STATUS_OK:
                return
            arenas = response.json()
            fake = Faker()

            while number_of_reservations != 0:
                arena = random.choice(arenas)
                rand_start_time = fake.date_time_between(start_date='now', end_date='+200d')
                hour_start = 13 if rand_start_time.hour > 13 else rand_start_time.hour

                response = self.client.post(hosts.arena_reservation + "/reservations/" + str(arena["id"]),
                                            json={
                                                "contactPersonId": contact_person_id,
                                                "dateTimeFrom": datetime.datetime(rand_start_time.year,
                                                                                  rand_start_time.month,
                                                                                  rand_start_time.day,
                                                                                  hour_start,
                                                                                  0, 0, 0).isoformat(),
                                                "dateTimeTo": datetime.datetime(rand_start_time.year,
                                                                                rand_start_time.month,
                                                                                rand_start_time.day,
                                                                                hour_start + 2,
                                                                                0, 0, 0).isoformat(),
                                                "reservationType": str(reservation_type)
                                            })
                if response.status_code == hosts.HTTP_STATUS_CREATED:
                    number_of_reservations -= 1
                else:
                    print("Time Collision with other Reservation")
                time.sleep(3)
