package cz.muni.fi.icehockeymanager.report;

import io.swagger.v3.oas.models.security.SecurityScheme;
import org.springdoc.core.customizers.OpenApiCustomizer;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Profile;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;


@SpringBootApplication
public class ReportApplication
{
    public static void main( String[] args )
    {
        SpringApplication.run(ReportApplication.class, args);
    }

    @Profile({"secure"})
    @Bean
    public SecurityFilterChain securityFilterChain(HttpSecurity httpSecurity) throws Exception {
        httpSecurity.csrf().disable().authorizeHttpRequests()
                .requestMatchers(new AntPathRequestMatcher("**", "POST")).authenticated()
                .requestMatchers(new AntPathRequestMatcher("**", "PUT")).authenticated()
                .requestMatchers(new AntPathRequestMatcher("**", "DELETE")).authenticated()
                .requestMatchers(new AntPathRequestMatcher("**", "PATCH")).authenticated()
                .requestMatchers(new AntPathRequestMatcher("**", "GET")).permitAll()
                .and().oauth2ResourceServer().jwt();
        return httpSecurity.build();
    }

    @Profile({"!secure"})
    @Bean
    public SecurityFilterChain securityFilterChainWithoutAuth(HttpSecurity httpSecurity) throws Exception {
        httpSecurity.csrf().disable().authorizeHttpRequests()
                .anyRequest().permitAll();
        return httpSecurity.build();
    }

    @Profile({"secure"})
    @Bean
    public OpenApiCustomizer openAPICustomizer() {
        return openApi -> {
            openApi.getComponents()
                .addSecuritySchemes("Bearer",
                    new SecurityScheme()
                        .type(SecurityScheme.Type.HTTP)
                        .scheme("bearer")
                        .description("Enter Bearer token")
                )
            ;
        };
    }
}
