package cz.muni.fi.icehockeymanager.report.facade;

import cz.muni.fi.icehockeymanager.report.api.create.CreateSubscriberDto;
import cz.muni.fi.icehockeymanager.report.api.edit.EditSubscriberDto;
import cz.muni.fi.icehockeymanager.report.api.view.SubscriberDto;
import cz.muni.fi.icehockeymanager.report.data.enums.SubscribeSubject;
import cz.muni.fi.icehockeymanager.report.data.repository.SubscriberRepository;
import cz.muni.fi.icehockeymanager.report.mappers.SubscriberMapper;
import cz.muni.fi.icehockeymanager.report.data.domain.Subscriber;
import cz.muni.fi.icehockeymanager.report.service.SubscriberService;
import jakarta.persistence.EntityNotFoundException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Optional;

@Service
@Transactional
public class SubscriberFacade {

    private final SubscriberService subscriberService;
    private final SubscriberRepository subscriberRepository;
    private final SubscriberMapper subscriberMapper;

    private final SubscriberDto defaultSubscriber;

    @Autowired
    public SubscriberFacade(SubscriberService subscriberService,
                            SubscriberMapper subscriberMapper,
                            SubscriberRepository subscriberRepository) {
        this.subscriberService = subscriberService;
        this.subscriberMapper = subscriberMapper;
        this.subscriberRepository = subscriberRepository;

        defaultSubscriber = new SubscriberDto();
        defaultSubscriber.setId(1L);
        defaultSubscriber.setName("Adam");
        defaultSubscriber.setMail("adam@gmail.com");
        defaultSubscriber.setPlayerRefId(1L);
        defaultSubscriber.setSubscribedSubjects(List.of(SubscribeSubject.GAME_CANCELLED));
    }

    @Transactional(readOnly = true)
    public SubscriberDto findById(Long id) {
        return subscriberMapper.mapToDto(subscriberService.findById(id));
    }

    @Transactional(readOnly = true)
    public Page<SubscriberDto> findAll(Pageable pageable) {
        return subscriberMapper.mapToPageDto(subscriberService.findAll(pageable));
    }

    public List<SubscriberDto> getAllSubscribers() {
        return subscriberMapper.mapToList(subscriberService.findAll());
    }

    public SubscriberDto getSubscriberById(Long subscriberId) {
        Optional<Subscriber> subscriber = subscriberRepository.findById(subscriberId);

        if (subscriber.isPresent()) {
            return subscriberMapper.mapToDto(subscriber.get());
        } else {
            throw new EntityNotFoundException("Subscriber not found with id: " + subscriberId);
        }
    }

    public SubscriberDto createSubscriber(CreateSubscriberDto createSubscriberDto) {
        Subscriber subscriber = subscriberMapper.mapCreateDtoToEntity(createSubscriberDto);
        return subscriberMapper.mapToDto(subscriberService.createSubscriber(subscriber));
    }

    public SubscriberDto updateSubscriber(Long id, SubscriberDto newSubscriberDto) {
        return subscriberService.updateSubscriber(id, newSubscriberDto);
    }

    public void deleteSubscriber(Long subscriberId) {
        subscriberRepository.deleteById(subscriberId);
    }

    public SubscriberDto changeSubscriberPreferences(Long id, EditSubscriberDto editSubscriberDto) {
        return subscriberService.changeSubscriberPreferences(id, editSubscriberDto);
    }
}
