package cz.muni.fi.icehockeymanager.report.rest;

import cz.muni.fi.icehockeymanager.report.api.edit.EditSubscriberDto;
import cz.muni.fi.icehockeymanager.report.api.view.SubscriberDto;
import cz.muni.fi.icehockeymanager.report.data.enums.SubscribeSubject;
import cz.muni.fi.icehockeymanager.report.facade.SubscriberFacade;
import cz.muni.fi.icehockeymanager.report.service.NotificationService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import org.springframework.beans.factory.annotation.Autowired;
import jakarta.validation.Valid;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/notify")
@SecurityRequirement(name = "Bearer")
public class NotificationRestController {
    private final SubscriberFacade subscriberFacade;
    private final NotificationService notificationService;

    @Autowired
    public NotificationRestController(SubscriberFacade subscriberFacade, NotificationService notificationService) {
        this.subscriberFacade = subscriberFacade;
        this.notificationService = notificationService;
    }

    @Operation(summary = "Notify a subscriber")
    @PostMapping("/{subscriberId}")
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<String> notifySubscriber(@PathVariable Long subscriberId, @Valid @RequestBody SubscribeSubject subscribeSubject) {
        if (notificationService.sendMail(subscriberFacade.getSubscriberById(subscriberId), subscribeSubject))
            return ResponseEntity.ok("Subscriber with id " + subscriberId + " notified successfully.");
        else
            return ResponseEntity.internalServerError().body("Subscriber with id " + subscriberId + " was not notified.");
    }

    @Operation(summary = "Notify subscribers")
    @PostMapping
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<String> notifySubscribers(@Valid @RequestBody SubscribeSubject subscribeSubject) {
        // Send notification to subscribed users about the event

        return ResponseEntity.ok("Notification sent successfully.");
    }

    @Operation(summary = "Change subscriber preferences")
    @PutMapping("/preferences/{subscriberId}")
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<SubscriberDto> changeSubscriberPreferences(@PathVariable Long subscriberId,
                                                                     @Valid @RequestBody EditSubscriberDto editSubscriberDto) {
        return ResponseEntity.ok(subscriberFacade.changeSubscriberPreferences(subscriberId, editSubscriberDto));
    }
}
