package cz.muni.fi.icehockeymanager.report.rest;

import cz.muni.fi.icehockeymanager.report.api.create.CreateSubscriberDto;
import cz.muni.fi.icehockeymanager.report.api.view.SubscriberDto;
import cz.muni.fi.icehockeymanager.report.facade.SubscriberFacade;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import jakarta.validation.Valid;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Optional;

@RestController
@RequestMapping("/subscribers")
@SecurityRequirement(name = "Bearer")
public class SubscriberRestController {
    private final SubscriberFacade subscriberFacade;

    @Autowired
    public SubscriberRestController(SubscriberFacade subscriberFacade) {
        this.subscriberFacade = subscriberFacade;
    }

    @Operation(summary = "Get a subscriber")
    @GetMapping("/{subscriberId}")
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<SubscriberDto> getSubscriber(@PathVariable Long subscriberId) {
        SubscriberDto subscriberDto = subscriberFacade.getSubscriberById(subscriberId);

        if (subscriberDto == null)
            return ResponseEntity.notFound().build();

        return ResponseEntity.ok(subscriberDto);
    }

    @Operation(summary = "Get all subscribers")
    @GetMapping
    @ResponseStatus(HttpStatus.OK)
    public ResponseEntity<List<SubscriberDto>> getSubscribers() {
        return ResponseEntity.ok(subscriberFacade.getAllSubscribers());
    }

    @Operation(summary = "Create a subscriber`")
    @PostMapping
    @ResponseStatus(HttpStatus.CREATED)
    public ResponseEntity<SubscriberDto> createSubscriber(@Valid @RequestBody CreateSubscriberDto createSubscriberDto) {
        return new ResponseEntity<>(subscriberFacade.createSubscriber(createSubscriberDto), HttpStatus.CREATED);
    }

    @Operation(summary = "Delete subscriber")
    @DeleteMapping("/{subscriberId}")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    public ResponseEntity<Void> deleteSubscriber(@PathVariable Long subscriberId) {
        SubscriberDto subscriberDto = subscriberFacade.getSubscriberById(subscriberId);

        if (subscriberDto == null)
            return ResponseEntity.notFound().build();

        subscriberFacade.deleteSubscriber(subscriberId);
        return ResponseEntity.noContent().build();
    }

    @Operation(summary = "Update subscriber")
    @PutMapping("/{subscriberId}")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    public ResponseEntity<SubscriberDto> updateSubscriber(@PathVariable Long subscriberId,
                                                          @Valid @RequestBody SubscriberDto subscriberDto) {
        SubscriberDto foundSubscriberDto = subscriberFacade.getSubscriberById(subscriberId);

        if (foundSubscriberDto == null)
            return ResponseEntity.notFound().build();

        return ResponseEntity.ok(subscriberFacade.updateSubscriber(subscriberId, subscriberDto));
    }
}
