package cz.muni.fi.icehockeymanager.report.service;

import cz.muni.fi.icehockeymanager.report.api.view.SubscriberDto;
import cz.muni.fi.icehockeymanager.report.data.enums.SubscribeSubject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.mail.MailException;
import org.springframework.mail.SimpleMailMessage;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.stereotype.Service;

@Service
public class NotificationService {
    private final JavaMailSender mailSender;
    private final Environment environment;

    @Autowired
    public NotificationService(JavaMailSender mailSender, Environment environment) {
        this.mailSender = mailSender;
        this.environment = environment;
    }

    public boolean sendMail(SubscriberDto subscriberDto, SubscribeSubject subscribeSubject) {
        SimpleMailMessage message = new SimpleMailMessage();
        message.setFrom(environment.getProperty("spring.mail.username"));
        message.setTo(subscriberDto.getMail());
        message.setSubject("Hockey notification");
        message.setText(ComposeMessage(subscribeSubject));

        try{
            mailSender.send(message);
        }
        catch (MailException e)
        {
            return false;
        }

        return true;
    }

    private String ComposeMessage(SubscribeSubject subscribeSubject)
    {
        return "An event " + subscribeSubject + " happened";
    }
}
