package cz.muni.fi.icehockeymanager.report.service;

import cz.muni.fi.icehockeymanager.report.api.edit.EditSubscriberDto;
import cz.muni.fi.icehockeymanager.report.api.view.SubscriberDto;
import cz.muni.fi.icehockeymanager.report.data.domain.Subscription;
import cz.muni.fi.icehockeymanager.report.data.enums.SubscribeSubject;
import cz.muni.fi.icehockeymanager.report.data.repository.SubscriberRepository;
import cz.muni.fi.icehockeymanager.report.exceptions.ResourceNotFoundException;
import cz.muni.fi.icehockeymanager.report.mappers.SubscriberMapper;
import cz.muni.fi.icehockeymanager.report.data.domain.Subscriber;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@Service
@Transactional
public class SubscriberService {

    private final SubscriberRepository subscriberRepository;
    private final SubscriberMapper subscriberMapper;

    @Autowired
    public SubscriberService(SubscriberRepository subscriberRepository, SubscriberMapper subscriberMapper) {
        this.subscriberRepository = subscriberRepository;
        this.subscriberMapper = subscriberMapper;
    }

    @Transactional(readOnly = true)
    public Subscriber findById(Long id) {
        return subscriberRepository.findById(id)
                .orElseThrow(() -> new ResourceNotFoundException("Person with id: " + id + " was not found."));
    }

    @Transactional(readOnly = true)
    public List<Subscriber> findAll() {
        return subscriberRepository.findAll();
    }

    @Transactional(readOnly = true)
    public Page<Subscriber> findAll(Pageable pageable) {
        return subscriberRepository.findAll(pageable);
    }

    public Subscriber createSubscriber(Subscriber subscriber) {
        return subscriberRepository.save(subscriber);
    }

    public SubscriberDto updateSubscriber(Long subscriberId, SubscriberDto newSubscriberDto) {
        Optional<Subscriber> subscriberOptional = subscriberRepository.findById(subscriberId);

        if (subscriberOptional.isPresent()) {
            Subscriber subscriber = subscriberMapper.mapToEntity(newSubscriberDto);
            subscriber.setId(subscriberId);
            return subscriberMapper.mapToDto(subscriberRepository.save(subscriber));
        } else {
            throw new RuntimeException("Subscriber not found with ID: " + subscriberId);
        }
    }

    public SubscriberDto changeSubscriberPreferences(Long id, EditSubscriberDto editSubscriberDto) {
        Optional<Subscriber> subscriberOptional = subscriberRepository.findById(id);

        if (subscriberOptional.isEmpty())
            return null;

        Subscriber subscriber = subscriberOptional.get();
        List<Subscription> subscriptions = new ArrayList<>();

        for (SubscribeSubject subject : editSubscriberDto.getSubscribedSubjects()) {
            Subscription subscription = new Subscription();
            subscription.setSubscribeSubject(subject);
            subscription.setDateAdded(LocalDate.now());
            subscription.setSubscriber(subscriber);
            subscriptions.add(subscription);
        }

        subscriber.setSubscriptions(subscriptions);
        return subscriberMapper.mapToDto(subscriber);
    }
}
