package cz.muni.fi.icehockeymanager.report;

import cz.muni.fi.icehockeymanager.report.api.edit.EditSubscriberDto;
import cz.muni.fi.icehockeymanager.report.api.view.SubscriberDto;
import cz.muni.fi.icehockeymanager.report.data.enums.SubscribeSubject;
import cz.muni.fi.icehockeymanager.report.facade.SubscriberFacade;
import cz.muni.fi.icehockeymanager.report.rest.NotificationRestController;

import cz.muni.fi.icehockeymanager.report.service.NotificationService;
import org.junit.jupiter.api.Test;
import static org.junit.jupiter.api.Assertions.assertEquals;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;

import static org.mockito.ArgumentMatchers.any;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import java.util.List;


@WebMvcTest(controllers = {NotificationRestController.class})
public class NotificationRestControllerTest {

    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private SubscriberFacade subscriberFacade;

    @MockBean
    private NotificationService notificationService;

    private final String SUBSCRIBER_1_JSON = "{\"id\":1,\"name\":\"John Doe\",\"mail\":\"john.doe@gmail.com\",\"playerRefId\":12345,\"subscribedSubjects\":[\"GAME_CANCELLED\",\"GAME_RESCHEDULED\"]}";
    private final SubscriberDto subscriber1Dto;

    public NotificationRestControllerTest() {
        subscriber1Dto = new SubscriberDto();

        subscriber1Dto.setId(1L);
        subscriber1Dto.setName("John Doe");
        subscriber1Dto.setMail("john.doe@gmail.com");
        subscriber1Dto.setPlayerRefId(12345L);
        subscriber1Dto.setSubscribedSubjects(List.of(SubscribeSubject.GAME_CANCELLED, SubscribeSubject.GAME_RESCHEDULED));
    }

    @Test
    public void testNotifySubscriber() throws Exception {
        Mockito.when(subscriberFacade.getSubscriberById(any(Long.class))).thenReturn(subscriber1Dto);
        Mockito.when(notificationService.sendMail(any(SubscriberDto.class), any(SubscribeSubject.class))).thenReturn(true);

        MvcResult response = mockMvc.perform(post("/notify/1")
                        .contentType("application/json")
                        .content("\"GAME_CANCELLED\""))
                .andExpect(status().isOk())
                .andReturn();

        assertEquals("Subscriber with id 1 notified successfully.", response.getResponse().getContentAsString());
    }

    @Test
    public void testNotifySubscribers() throws Exception {
        Mockito.when(notificationService.sendMail(any(SubscriberDto.class), any(SubscribeSubject.class))).thenReturn(true);

        MvcResult response = mockMvc.perform(post("/notify")
                        .contentType("application/json")
                        .content("\"GAME_CANCELLED\"")
                )
                .andExpect(status().isOk())
                .andReturn();

        assertEquals("Notification sent successfully.", response.getResponse().getContentAsString());
    }

    @Test
    public void testChangeSubscriberPreferences() throws Exception {
        Mockito.when(subscriberFacade.changeSubscriberPreferences(any(Long.class), any(EditSubscriberDto.class)))
                .thenReturn(subscriber1Dto);

        MvcResult response = mockMvc.perform(put("/notify/preferences/1")
                        .contentType("application/json")
                        .content("{\"subscribedSubjects\":[\"GAME_CANCELLED\", \"GAME_RESCHEDULED\"]}")
                )
                .andExpect(status().isOk())
                .andReturn();

        assertEquals(SUBSCRIBER_1_JSON, response.getResponse().getContentAsString());
    }
}