package cz.muni.fi.icehockeymanager.report;

import cz.muni.fi.icehockeymanager.report.data.domain.Subscriber;
import cz.muni.fi.icehockeymanager.report.service.NotificationService;
import cz.muni.fi.icehockeymanager.report.service.SubscriberService;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;

import java.util.List;

@SpringBootTest
public class SubscriberServiceTest {
    @MockBean
    private NotificationService notificationService;

    @Autowired
    private final SubscriberService subscriberService;

    @Autowired
    public SubscriberServiceTest(SubscriberService subscriberService) {
        this.subscriberService = subscriberService;
    }

    @Test
    public void testFindById() {
        Subscriber subscriber = subscriberService.findById(1L);
        assert subscriber.getName().equals("John Doe");
    }

    @Test
    public void testFindAll() {
        List<Subscriber> subscribers = subscriberService.findAll();
        assert subscribers.size() == 2;
        assert subscribers.get(0).getName().equals("John Doe");
        assert subscribers.get(1).getName().equals("Jane Smith");
    }

    @Test
    public void testFindAllPageable() {
        Page<Subscriber> subscribers = subscriberService.findAll(PageRequest.of(0, 10));
        assert subscribers.getTotalElements() == 2;
        assert subscribers.getContent().get(0).getName().equals("John Doe");
        assert subscribers.getContent().get(1).getName().equals("Jane Smith");
    }

    @Test
    public void testCreateSubscriber() {
        Subscriber subscriber = new Subscriber();
        subscriber.setName("Adam");
        subscriber.setMail("name@email");
        subscriber.setPlayerRefId(1L);

        subscriberService.createSubscriber(subscriber);

        Subscriber createdSubscriber = subscriberService.findById(3L);
        assert createdSubscriber.getName().equals("Adam");
    }
}
